/**
 *****************************************************************************
   @addtogroup comp 
   @{
   @file       CmpLib.c
   @brief      Set of comparator peripheral functions.

   @internal   001 @endinternal
   @version    V1.0
   @author     ADI
   @date       August 2015
   @par Revision History:
   - V0.1, October 2013: initial version.
   - V0.2, January 2014: Fixed Doxygen parameters
   - V1.0, August 2015: Full release


All files provided by ADI, including this file, are
provided as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include "CmpLib.h"

/**
	@brief int CmpCfg(int iInNeg, int iSpd, int iHys)	
			==========Sets speed and hysteresis.
	@param iInNeg :{AFECOMP_INNEG_AVDD2,AFECOMP_INNEG_AIN5,AFECOMP_INNEG_DAC7}
		-0 or AFECOMP_INNEG_AVDD2 for AVDD/2 as negative input.
		-0x80 or AFECOMP_INNEG_AIN5 for AIN6 as negative input.
		-0x100 or AFECOMP_INNEG_DAC7 for DAC7 as negative input.
	@param iSpd :{AFECOMP_SPEED_SPEED6, AFECOMP_SPEED_SPEED3, 
                AFECOMP_SPEED_SPEED2,AFECOMP_SPEED_SPEED1}
		-0 or AFECOMP_SPEED_SPEED6 for 6uS response time.
		-1 or AFECOMP_SPEED_SPEED3 for 3.25uS response time.
		-2 or AFECOMP_SPEED_SPEED2 for 1.75uS response time.
		-3 or AFECOMP_SPEED_SPEED1 for 1uS response time.
	@param iHys :{AFECOMP_HYS_DIS,AFECOMP_HYS_EN}
		- 0 or AFECOMP_HYS_DIS removes hysteresis.
		- Not 0 or AFECOMP_HYS_EN inserts hysteresis.
	@return value written to AFECOMP.
**/

int CmpCfg(int iInNeg, int iSpd, int iHys)
	{
    int i1;
    
    i1 = pADI_AFE->AFECOMP;
    i1 &= ~(AFECOMP_INNEG_MSK|AFECOMP_SPEED_MSK|AFECOMP_HYS_MSK);
    i1 |= iSpd&AFECOMP_SPEED_MSK;
    i1 |= iInNeg&AFECOMP_INNEG_MSK;
    if(iHys) i1 |= AFECOMP_HYS_EN;
    pADI_AFE->AFECOMP = i1;
	return i1;
	}

/**
	@brief int CmpOn(int iOn, int iInvert)	
			==========Powers up and enables comparator and chooses direction.
	@param iOn :{AFECOMP_EN_DIS, AFECOMP_EN_EN}
		-0 or AFECOMP_EN_DIS to power off comparator.
		-not 0 or AFECOMP_EN_EN to power up and enable comparator.
	@param iInvert :{AFECOMP_INV_NO, AFECOMP_INV_YES}
		-0 or AFECOMP_INV_NO for output high if +ve input above -ve input.
		-not 0 or AFECOMP_INV_YES for output high if +ve input below -ve input.
	@return 1.
**/

int CmpOn(int iOn, int iInvert)
	{
    int i1;
    
    i1 = pADI_AFE->AFECOMP;
    i1 &= ~(AFECOMP_OUT_MSK | AFECOMP_EN_MSK) | AFECOMP_INV_MSK;
    if(iOn)  i1 |= (AFECOMP_OUT_EN | AFECOMP_EN_EN);
//    if(iOn)  i1 |= (AFECOMP_EN_EN);
    i1 |= iInvert&AFECOMP_INV_MSK;
    pADI_AFE->AFECOMP = i1;
	return 1;
	}

