/**
 *****************************************************************************
   @addtogroup dma 
   @{
   @file    DmaLib.c
   @brief   Set of DMA peripheral functions.
   - DmaBase(), DmaSet(), DmaClr(), DmaSta() and DmaErr() apply to all DMA channels together.
   
   @internal   001 @endinternal
   @version    V1.1
   @author     ADI
   @date       October 2015
   @par Revision History:
   - V0.1, July 2013: initial version.
   - V0.2, September 2013: DmaStructPtrOutSetup() now needs a const pointer,
                           Dma flash support
   - V0.3, May 2014: Fixed doxygen comments specifying the range of parameters
   - V0.4, November 2014: Added AdcDmaReadSetup function for ADC DMA                        
   - V1.0, April 2015: Release
   - V1.1, October 2015: Corrected doxygen comments for AdcDmaReadSetup

All files provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.
**/
#include <stdio.h>
#include <string.h>
#include "DmaLib.h"

// Define dmaChanDesc as an array of descriptors aligned to the required 
// boundary for all supported compilers. It was not possible to do something
// in a macro here because the #pragma for IAR cannot be put in a #define.
// 
// Support primary and alternate here
#define DMACHAN_NUM ((INT_NUM_DMA_LAST - INT_NUM_DMA_FIRST) + 1)

#if defined(__ARMCC_VERSION) || defined(__GNUC__)
DmaDesc dmaChanDesc     [CCD_SIZE * 2] __attribute__ ((aligned (DMACHAN_DSC_ALIGN)));      
#endif

#ifdef __ICCARM__
#pragma data_alignment=(DMACHAN_DSC_ALIGN)
DmaDesc dmaChanDesc     [CCD_SIZE * 2];
#endif



/**
   @brief DmaDesc * Dma_GetDescriptor(unsigned int iChan);
         ========== Returns the Primary or Alternate structure descriptor of the 
         specified channel.
   @param iChan :{SPI0TX_C,SPI0RX_C,SPI1TX_C,SPI1RX_C,UARTTX_C,UARTRX_C,
         I2C0STX_C,I2C0SRX_C,I2C0M_C,I2C1STX_C,I2C1SRX_C,I2C1M_C,ADC_C,Flash_C,
         SPI0TX_C+ALTERNATE,SPI0RX_C+ALTERNATE,SPI1TX_C+ALTERNATE,SPI1RX_C+ALTERNATE,
         UARTTX_C+ALTERNATE,UARTRX_C+ALTERNATE,I2C0STX_C+ALTERNATE,I2C0SRX_C+ALTERNATE,
         I2C0M_C+ALTERNATE,I2C1STX_C+ALTERNATE,I2C1SRX_C+ALTERNATE,I2C1M_C+ALTERNATE,
         ADC_C+ALTERNATE,Flash_C+ALTERNATE}
    - 0 or SPI0TX_C   for the SPI0 Transmit channel       - primary structure
    - 1 or SPI0RX_C   for the SPI0 Receive channel        - primary structure
    - 2 or SPI1TX_C   for the SPI1 Transmit channel       - primary structure
    - 3 or SPI1RX_C   for the SPI1 Receive channel        - primary structure
    - 4 or UARTTX_C   for the UART Transmit channel       - primary structure
    - 5 or UARTRX_C   for the UART Receive channel        - primary structure
    - 6 or I2C0STX_C  for the I2C0 Slave transmit channel - primary structure 
    - 7 or I2C0SRX_C  for the I2C0 Slave receive channel  - primary structure 
    - 8 or I2C0M_C    for the I2C0 Master channel         - primary structure
    - 9 or I2C1STX_C  for the I2C1 Slave transmit channel - primary structure 
    - 10 or I2C1SRX_C for the I2C1 Slave receive channel  - primary structure
    - 11 or I2C1M_C   for the I2C1 Master channel         - primary structure
    - 12 or ADC_C     for the ADC output channel          - primary structure 
    - 13 or Flash_C   for the Flash channel               - primary structure 
    - 16 or SPI0TX_C+ALTERNATE   for the SPI0 Transmit channel       - ALTERNATE structure
    - 17 or SPI0RX_C+ALTERNATE   for the SPI0 Receive channel        - ALTERNATE structure
    - 18 or SPI1TX_C+ALTERNATE   for the SPI1 Transmit channel       - ALTERNATE structure
    - 19 or SPI1RX_C+ALTERNATE   for the SPI1 Receive channel        - ALTERNATE structure
    - 20 or UARTTX_C+ALTERNATE   for the UART Transmit channel       - ALTERNATE structure 
    - 21 or UARTRX_C+ALTERNATE   for the UART Receive channel        - ALTERNATE structure 
    - 22 or I2C0STX_C+ALTERNATE  for the I2C0 Slave transmit channel - ALTERNATE structure 
    - 23 or I2C0SRX_C+ALTERNATE  for the I2C0 Slave receive channel  - ALTERNATE structure 
    - 24 or I2C0M_C+ALTERNATE    for the I2C0 Master channel         - ALTERNATE structure 
    - 25 or I2C1STX_C+ALTERNATE  for the I2C1 Slave transmit channel - ALTERNATE structure 
    - 26 or I2C1SRX_C+ALTERNATE  for the I2C1 Slave receive channel  - ALTERNATE structure 
    - 27 or I2C1M_C+ALTERNATE    for the I2C1 Master channel         - ALTERNATE structure 
    - 28 or ADC_C+ALTERNATE      for the ADC output channel          - ALTERNATE structure 
    - 29 or Flash_C+ALTERNATE    for the Flash channel               - ALTERNATE structure 
   @return 1.
	
**/

DmaDesc * Dma_GetDescriptor(unsigned int iChan)
{
   if (iChan < (CCD_SIZE*2))
   {
      return &(dmaChanDesc[iChan ]);
   } 
   else
      return 0x0;
}

/**
	@brief int DmaBase(void)
			========== Sets the Address of DMA Data base pointer.
	@return 1.
**/
int DmaBase(void)
{
   volatile unsigned int uiBasPtr = 0;
   
   memset(dmaChanDesc,0x0,sizeof(dmaChanDesc));       // Clear all the DMA descriptors (individual blocks will update their  descriptors
   uiBasPtr = (unsigned int)&dmaChanDesc;             // Setup the DMA base pointer.
   pADI_DMA->DMACFG = 1;                              // Enable DMA controller
   pADI_DMA->DMAPDBPTR = uiBasPtr;
   return 1;
	}
/**
   @brief int DmaSet(int iMask, int iEnable, int iAlt, int iPriority)
         ========== Controls Mask, Primary Enable, Alternate enable and priority enable bits
   @param iMask :{0|SPI0TX_B|SPI0RX_B|SPI1TX_B|SPI1RX_B|UARTTX_B|UARTRX_B
                   |I2C0STX_B|I2C0SRX_B|I2C0M_B|I2C1STX_B|I2C1SRX_B|I2C1M_B
                   |ADC_B|Flash_B}
      - Select a combination of peripheral mask bits to be set.
      - 0 = to Mask no DMA source
      - 0x1 or SPI0TX_B    to mask SPI0 Transmit 
      - 0x2 or SPI0RX_B    to mask SPI0 Receive
      - 0x4 or SPI1TX_B    to mask SPI1 Transmit 
      - 0x8 or SPI1RX_B    to mask SPI1 Receive
      - 0x10 or UARTTX_B   to mask UART Transmit
      - 0x20 or UARTRX_B   to mask UART Receive
      - 0x40 or I2C0STX_B  to mask I2C0 Slave Transmit
      - 0x80 or I2C0SRX_B  to mask I2C0 Slave Receive
      - 0x100 or I2C0M_B   to mask I2C0 Master
      - 0x200 or I2C1STX_B to mask I2C1 Slave Transmit
      - 0x400 or I2C1SRX_B to mask I2C1 Slave Receive
      - 0x800 or I2C1M_B   to mask I2C1 Master
      - 0x1000 or ADC_B    to mask ADC
      - 0x2000 OR Flash_B  to mask Flash
   @param iEnable :{0|SPI0TX_B|SPI0RX_B|SPI1TX_B|SPI1RX_B|UARTTX_B|UARTRX_B
                     |I2C0STX_B|I2C0SRX_B|I2C0M_B|I2C1STX_B|I2C1SRX_B|I2C1M_B
                     |ADC_B|Flash_B}
      - Select a combination of peripheral enable bits to be set.
      - 0 = to enable no DMA source
      - 0x1 or SPI0TX_B    to enable SPI0 Transmit 
      - 0x2 or SPI0RX_B    to enable SPI0 Receive
      - 0x4 or SPI1TX_B    to enable SPI1 Transmit 
      - 0x8 or SPI1RX_B    to enable SPI1 Receive
      - 0x10 or UARTTX_B   to enable UART Transmit
      - 0x20 or UARTRX_B   to enable UART Receive
      - 0x40 or I2C0STX_B  to enable I2C0 Slave Transmit
      - 0x80 or I2C0SRX_B  to enable I2C0 Slave Receive
      - 0x100 or I2C0M_B   to enable I2C0 Master
      - 0x200 or I2C1STX_B to enable I2C1 Slave Transmit
      - 0x400 or I2C1SRX_B to enable I2C1 Slave Receive
      - 0x800 or I2C1M_B   to enable I2C1 Master
      - 0x1000 or ADC_B    to enable ADC
      - 0x2000 OR Flash_B  to enable Flash
   @param iAlt :{0|SPI0TX_B|SPI0RX_B|SPI1TX_B|SPI1RX_B|UARTTX_B|UARTRX_B
                  |I2C0STX_B|I2C0SRX_B|I2C0M_B|I2C1STX_B|I2C1SRX_B|I2C1M_B
                  |ADC_B|Flash_B}
      - Select a combination of peripheral alternate bits to be set.
      - 0 = to set alternate no DMA source
      - 0x1 or SPI0TX_B    to set alternate SPI0 Transmit 
      - 0x2 or SPI0RX_B    to set alternate SPI0 Receive
      - 0x4 or SPI1TX_B    to set alternate SPI1 Transmit 
      - 0x8 or SPI1RX_B    to set alternate SPI1 Receive
      - 0x10 or UARTTX_B   to set alternate UART Transmit
      - 0x20 or UARTRX_B   to set alternate UART Receive
      - 0x40 or I2C0STX_B  to set alternate I2C0 Slave Transmit
      - 0x80 or I2C0SRX_B  to set alternate I2C0 Slave Receive
      - 0x100 or I2C0M_B   to set alternate I2C0 Master
      - 0x200 or I2C1STX_B to set alternate I2C1 Slave Transmit
      - 0x400 or I2C1SRX_B to set alternate I2C1 Slave Receive
      - 0x800 or I2C1M_B   to set alternate I2C1 Master
      - 0x1000 or ADC_B    to set alternate ADC
      - 0x2000 OR Flash_B  to set alternate Flash
   @param iPriority :{0|SPI0TX_B|SPI0RX_B|SPI1TX_B|SPI1RX_B|UARTTX_B|UARTRX_B
                       |I2C0STX_B|I2C0SRX_B|I2C0M_B|I2C1STX_B|I2C1SRX_B|I2C1M_B
                       |ADC_B|Flash_B}
      - Select a combination of peripheral priority bits to be set.
      - 0 to leave priority level of all channels as they are
      - 0x1 or SPI0TX_B    to high prioritise SPI0 Transmit 
      - 0x2 or SPI0RX_B    to high prioritise SPI0 Receive
      - 0x4 or SPI1TX_B    to high prioritise SPI1 Transmit 
      - 0x8 or SPI1RX_B    to high prioritise SPI1 Receive
      - 0x10 or UARTTX_B   to high prioritise UART Transmit
      - 0x20 or UARTRX_B   to high prioritise UART Receive
      - 0x40 or I2C0STX_B  to high prioritise I2C0 Slave Transmit
      - 0x80 or I2C0SRX_B  to high prioritise I2C0 Slave Receive
      - 0x100 or I2C0M_B   to high prioritise I2C0 Master
      - 0x200 or I2C1STX_B to high prioritise I2C1 Slave Transmit
      - 0x400 or I2C1SRX_B to high prioritise I2C1 Slave Receive
      - 0x800 or I2C1M_B   to high prioritise I2C1 Master
      - 0x1000 or ADC_B    to high prioritise ADC
      - 0x2000 OR Flash_B  to high prioritise Flash    
   @return 1.
**/
int DmaSet(int iMask, int iEnable, int iAlt, int iPriority)
	{
	pADI_DMA->DMARMSKSET = iMask;
	pADI_DMA->DMAENSET = iEnable;
	pADI_DMA->DMAALTSET = iAlt;
	pADI_DMA->DMAPRISET = iPriority;
	return 1;
	}

/**
   @brief int DmaClr(int iMask, int iDisable, int iAlt, int iPriority)
			========== Controls Mask, Primary Enable, Alternate enable and priority Clear bits
   @param iMask :{0|SPI0TX_B|SPI0RX_B|SPI1TX_B|SPI1RX_B|UARTTX_B|UARTRX_B
                   |I2C0STX_B|I2C0SRX_B|I2C0M_B|I2C1STX_B|I2C1SRX_B|I2C1M_B
                   |ADC_B|Flash_B}
      - Select a combination of peripheral mask bits to be cleared.
      - 0 = to disable no Mask bit
      - 0x1 or SPI0TX_B    to disable mask SPI0 Transmit 
      - 0x2 or SPI0RX_B    to disable mask SPI0 Receive
      - 0x4 or SPI1TX_B    to disable mask SPI1 Transmit 
      - 0x8 or SPI1RX_B    to disable mask SPI1 Receive
      - 0x10 or UARTTX_B   to disable mask UART Transmit
      - 0x20 or UARTRX_B   to disable mask UART Receive
      - 0x40 or I2C0STX_B  to disable mask I2C0 Slave Transmit
      - 0x80 or I2C0SRX_B  to disable mask I2C0 Slave Receive
      - 0x100 or I2C0M_B   to disable mask I2C0 Master
      - 0x200 or I2C1STX_B to disable mask I2C1 Slave Transmit
      - 0x400 or I2C1SRX_B to disable mask I2C1 Slave Receive
      - 0x800 or I2C1M_B   to disable mask I2C1 Master
      - 0x1000 or ADC_B    to disable mask ADC
      - 0x2000 OR Flash_B  to disable mask Flash
   @param iDisable :{0|SPI0TX_B|SPI0RX_B|SPI1TX_B|SPI1RX_B|UARTTX_B|UARTRX_B
                      |I2C0STX_B|I2C0SRX_B|I2C0M_B|I2C1STX_B|I2C1SRX_B|I2C1M_B
                      |ADC_B|Flash_B}
      - Select a combination of peripheral enable bits to be cleared.
      - 0 = to disable no DMA source
      - 0x1 or SPI0TX_B    to disable SPI0 Transmit 
      - 0x2 or SPI0RX_B    to disable SPI0 Receive
      - 0x4 or SPI1TX_B    to disable SPI1 Transmit 
      - 0x8 or SPI1RX_B    to disable SPI1 Receive
      - 0x10 or UARTTX_B   to disable UART Transmit
      - 0x20 or UARTRX_B   to disable UART Receive
      - 0x40 or I2C0STX_B  to disable I2C0 Slave Transmit
      - 0x80 or I2C0SRX_B  to disable I2C0 Slave Receive
      - 0x100 or I2C0M_B   to disable I2C0 Master
      - 0x200 or I2C1STX_B to disable I2C1 Slave Transmit
      - 0x400 or I2C1SRX_B to disable I2C1 Slave Receive
      - 0x800 or I2C1M_B   to disable I2C1 Master
      - 0x1000 or ADC_B    to disable ADC
      - 0x2000 OR Flash_B  to disable Flash
   @param iAlt :{0|SPI0TX_B|SPI0RX_B|SPI1TX_B|SPI1RX_B|UARTTX_B|UARTRX_B
                  |I2C0STX_B|I2C0SRX_B|I2C0M_B|I2C1STX_B|I2C1SRX_B|I2C1M_B
                  |ADC_B|Flash_B}
      - Select a combination of peripheral alternate bits to be cleared.
      - 0 = to enable no DMA source
      - 0x1 or SPI0TX_B    to clear alternate SPI0 Transmit 
      - 0x2 or SPI0RX_B    to clear alternate SPI0 Receive
      - 0x4 or SPI1TX_B    to clear alternate SPI1 Transmit 
      - 0x8 or SPI1RX_B    to clear alternate SPI1 Receive
      - 0x10 or UARTTX_B   to clear alternate UART Transmit
      - 0x20 or UARTRX_B   to clear alternate UART Receive
      - 0x40 or I2C0STX_B  to clear alternate I2C0 Slave Transmit
      - 0x80 or I2C0SRX_B  to clear alternate I2C0 Slave Receive
      - 0x100 or I2C0M_B   to clear alternate I2C0 Master
      - 0x200 or I2C1STX_B to clear alternate I2C1 Slave Transmit
      - 0x400 or I2C1SRX_B to clear alternate I2C1 Slave Receive
      - 0x800 or I2C1M_B   to clear alternate I2C1 Master
      - 0x1000 or ADC_B    to clear alternate ADC
      - 0x2000 OR Flash_B  to clear alternate Flash
   @param iPriority :{0|SPI0TX_B|SPI0RX_B|SPI1TX_B|SPI1RX_B|UARTTX_B|UARTRX_B
                       |I2C0STX_B|I2C0SRX_B|I2C0M_B|I2C1STX_B|I2C1SRX_B|I2C1M_B
                       |ADC_B|Flash_B}
      - Select a combination of peripheral priority bits to default level.
      - 0 to leave priority level of all channels as they are
      - 0x1 or SPI0TX_B    to default prioritise SPI0 Transmit 
      - 0x2 or SPI0RX_B    to default prioritise SPI0 Receive
      - 0x4 or SPI1TX_B    to default prioritise SPI1 Transmit 
      - 0x8 or SPI1RX_B    to default prioritise SPI1 Receive
      - 0x10 or UARTTX_B   to default prioritise UART Transmit
      - 0x20 or UARTRX_B   to default prioritise UART Receive
      - 0x40 or I2C0STX_B  to default prioritise I2C0 Slave Transmit
      - 0x80 or I2C0SRX_B  to default prioritise I2C0 Slave Receive
      - 0x100 or I2C0M_B   to default prioritise I2C0 Master
      - 0x200 or I2C1STX_B to default prioritise I2C1 Slave Transmit
      - 0x400 or I2C1SRX_B to default prioritise I2C1 Slave Receive
      - 0x800 or I2C1M_B   to default prioritise I2C1 Master
      - 0x1000 or ADC_B    to default prioritise ADC
      - 0x2000 OR Flash_B  to default prioritise Flash    
   @return 1.
**/
int DmaClr(int iMask, int iDisable, int iAlt, int iPriority)
	{
	pADI_DMA->DMARMSKCLR = iMask;
	pADI_DMA->DMAENCLR = iDisable;
	pADI_DMA->DMAALTCLR = iAlt;
	pADI_DMA->DMAPRICLR = iPriority;
	return 1;
	}


/**
   @brief int DmaSta(void)
      ========== Reads the status of the DMA controller.
   @return value of DMASTA	/n
      bit 0 - enabled	/n
      bits [7:4]
      - 0 = Idle
      - 1 = Reading channel controller data
      - 2 = Reading source data end pointer
      - 3 = Reading destination end pointer
      - 4 = Reading source data
      - 5 = Writing destination data
      - 6 = Waiting for DMArequest to clear
      - 7 = Writing channel controller data
      - 8 = Stalled
      - 9 = Done
      - 10 = Scatter gather transition.
      bits [20:16]
      - number of DMA channels available -1
		
**/
int DmaSta(void)
	{
	return pADI_DMA->DMASTA;
	}

/**
   @brief int DmaErr(int iErrClr)
         ========== Reads and optionally clears DMA error bit.
   @param iErrClr :{DMA_ERR_RD,DMA_ERR_CLR}
      - 0 or DMA_ERR_RD only reads error bit.
      - 1 or DMA_ERR_CLR reads then clears error bit.
   @return value of DMAERRCLR
      - 0 - flags no error
      - 1 - flags error.
**/
int DmaErr(int iErrClr)
	{
	if(iErrClr)	
			pADI_DMA->DMAERRCLR = 1;
	return pADI_DMA->DMAERRCLR;
	}


/**
   @brief int DmaPeripheralStructSetup(int iChan, int iCfg)	
         ==========Sets up DMA config structure for the required channel
            
   @param iChan :{SPI0TX_C,SPI0RX_C,SPI1TX_C,SPI1RX_C,UARTTX_C,UARTRX_C,
         I2C0STX_C,I2C0SRX_C,I2C0M_C,I2C1STX_C,I2C1SRX_C,I2C1M_C,ADC_C,Flash_C,
         SPI0TX_C+ALTERNATE,SPI0RX_C+ALTERNATE,SPI1TX_C+ALTERNATE,SPI1RX_C+ALTERNATE,
         UARTTX_C+ALTERNATE,UARTRX_C+ALTERNATE,I2C0STX_C+ALTERNATE,I2C0SRX_C+ALTERNATE,
         I2C0M_C+ALTERNATE,I2C1STX_C+ALTERNATE,I2C1SRX_C+ALTERNATE,I2C1M_C+ALTERNATE,
         ADC_C+ALTERNATE,Flash_C+ALTERNATE}
    - 0 or SPI0TX_C   for the SPI0 Transmit channel       - primary structure
    - 1 or SPI0RX_C   for the SPI0 Receive channel        - primary structure
    - 2 or SPI1TX_C   for the SPI1 Transmit channel       - primary structure
    - 3 or SPI1RX_C   for the SPI1 Receive channel        - primary structure
    - 4 or UARTTX_C   for the UART Transmit channel       - primary structure
    - 5 or UARTRX_C   for the UART Receive channel        - primary structure
    - 6 or I2C0STX_C  for the I2C0 Slave transmit channel - primary structure 
    - 7 or I2C0SRX_C  for the I2C0 Slave receive channel  - primary structure 
    - 8 or I2C0M_C    for the I2C0 Master channel         - primary structure
    - 9 or I2C1STX_C  for the I2C1 Slave transmit channel - primary structure 
    - 10 or I2C1SRX_C for the I2C1 Slave receive channel  - primary structure
    - 11 or I2C1M_C   for the I2C1 Master channel         - primary structure
    - 12 or ADC_C     for the ADC output channel          - primary structure 
    - 13 or Flash_C   for the Flash channel               - primary structure 
    - 16 or SPI0TX_C+ALTERNATE   for the SPI0 Transmit channel       - ALTERNATE structure
    - 17 or SPI0RX_C+ALTERNATE   for the SPI0 Receive channel        - ALTERNATE structure
    - 18 or SPI1TX_C+ALTERNATE   for the SPI1 Transmit channel       - ALTERNATE structure
    - 19 or SPI1RX_C+ALTERNATE   for the SPI1 Receive channel        - ALTERNATE structure
    - 20 or UARTTX_C+ALTERNATE   for the UART Transmit channel       - ALTERNATE structure 
    - 21 or UARTRX_C+ALTERNATE   for the UART Receive channel        - ALTERNATE structure 
    - 22 or I2C0STX_C+ALTERNATE  for the I2C0 Slave transmit channel - ALTERNATE structure 
    - 23 or I2C0SRX_C+ALTERNATE  for the I2C0 Slave receive channel  - ALTERNATE structure 
    - 24 or I2C0M_C+ALTERNATE    for the I2C0 Master channel         - ALTERNATE structure 
    - 25 or I2C1STX_C+ALTERNATE  for the I2C1 Slave transmit channel - ALTERNATE structure 
    - 26 or I2C1SRX_C+ALTERNATE  for the I2C1 Slave receive channel  - ALTERNATE structure 
    - 27 or I2C1M_C+ALTERNATE    for the I2C1 Master channel         - ALTERNATE structure 
    - 28 or ADC_C+ALTERNATE      for the ADC output channel          - ALTERNATE structure 
    - 29 or Flash_C+ALTERNATE    for the Flash channel               - ALTERNATE structure 
   @param iCfg :{DMA_DSTINC_BYTE|DMA_DSTINC_HWORD|DMA_DSTINC_WORD|DMA_DSTINC_NO|
                 DMA_SRCINC_BYTE|DMA_SRCINC_HWORD|DMA_SRCINC_WORD|DMA_SRCINC_NO|
                 DMA_SIZE_BYTE|DMA_SIZE_HWORD|DMA_SIZE_WORD}
   - Choose one of DMA_DSTINC_BYTE,DMA_DSTINC_HWORD,DMA_DSTINC_WORD,DMA_DSTINC_NO for destination address increment
   - Choose one of DMA_SRCINC_BYTE,DMA_SRCINC_HWORD,DMA_SRCINC_WORD,DMA_SRCINC_NO for source address increment
   - Choose one of DMA_SIZE_BYTE (byte),DMA_SIZE_HWORD (half-word),DMA_SIZE_WORD(word) for source data size

   @return 1:

**/
int DmaPeripheralStructSetup(int iChan, int iCfg)
{
	DmaDesc Desc;
   Desc.ctrlCfg.ctrlCfgVal = 0;
   Desc.destEndPtr = 0;
   Desc.reserved4Bytes = 0;
   Desc.srcEndPtr = 0;

    // Common configuration of all the descriptors used here
  //  Desc.ctrlCfg.Bits.cycle_ctrl       = (iCfg & 0x7);
   Desc.ctrlCfg.Bits.next_useburst    = 0x0;
   Desc.ctrlCfg.Bits.r_power          = 0;  
   Desc.ctrlCfg.Bits.src_prot_ctrl    = 0x0;
   Desc.ctrlCfg.Bits.dst_prot_ctrl    = 0x0;
   Desc.ctrlCfg.Bits.src_size         = ((iCfg & 0x3000000)>>24);
   Desc.ctrlCfg.Bits.dst_size         = ((iCfg & 0x3000000)>>24);

   Desc.ctrlCfg.Bits.n_minus_1        = 0;
//     Desc.ctrlCfg.Bits.n_minus_1        = 16;
   Desc.ctrlCfg.Bits.src_inc          = ((iCfg & 0xC000000)>>26);
   Desc.ctrlCfg.Bits.dst_inc          = ((iCfg & 0xC0000000)>>30);

//     Desc.ctrlCfg.Bits.src_size         = 1;
//     Desc.ctrlCfg.Bits.dst_size         = 1;

//     Desc.ctrlCfg.Bits.src_inc          = 1;
//     Desc.ctrlCfg.Bits.dst_inc          = 3;

   *Dma_GetDescriptor(iChan) = Desc;     // structure

   return 1;	
}
/**
   @brief int DmaStructPtrOutSetup(int iChan, int iNumVals, const unsigned char *pucTX_DMA)	
            ==========For DMA operations where the destination is fixed (peripheral register is fixed)
   @param iChan :{SPI0TX_C,SPI1TX_C,UARTTX_C,
         I2C0STX_C,I2C0M_C,I2C1STX_C,I2C1M_C,Flash_C,
         SPI0TX_C+ALTERNATE,SPI1TX_C+ALTERNATE,
         UARTTX_C+ALTERNATE,I2C0STX_C+ALTERNATE,
         I2C0M_C+ALTERNATE,I2C1STX_C+ALTERNATE,I2C1M_C+ALTERNATE,
         Flash_C+ALTERNATE}
    - 0 or SPI0TX_C   for the SPI0 Transmit channel       - primary structure
    - 2 or SPI1TX_C   for the SPI1 Transmit channel       - primary structure
    - 4 or UARTTX_C   for the UART Transmit channel       - primary structure
    - 6 or I2C0STX_C  for the I2C0 Slave transmit channel - primary structure 
    - 8 or I2C0M_C    for the I2C0 Master channel         - primary structure
    - 9 or I2C1STX_C  for the I2C1 Slave transmit channel - primary structure 
    - 11 or I2C1M_C   for the I2C1 Master channel         - primary structure
    - 13 or Flash_C   for the Flash channel               - primary structure 
    - 16 or SPI0TX_C+ALTERNATE   for the SPI0 Transmit channel       - ALTERNATE structure
    - 18 or SPI1TX_C+ALTERNATE   for the SPI1 Transmit channel       - ALTERNATE structure
    - 20 or UARTTX_C+ALTERNATE   for the UART Transmit channel       - ALTERNATE structure 
    - 22 or I2C0STX_C+ALTERNATE  for the I2C0 Slave transmit channel - ALTERNATE structure 
    - 24 or I2C0M_C+ALTERNATE    for the I2C0 Master channel         - ALTERNATE structure 
    - 25 or I2C1STX_C+ALTERNATE  for the I2C1 Slave transmit channel - ALTERNATE structure 
    - 27 or I2C1M_C+ALTERNATE    for the I2C1 Master channel         - ALTERNATE structure 
    - 29 or Flash_C+ALTERNATE    for the Flash channel               - ALTERNATE structure 
   @param iNumVals :{1-1024}
   - 1 to 1024. Number of values to transfer
   @param *pucTX_DMA :{0-0xFFFFFFFF}
   - Pass Source pointer  address for DMA transfers
   @return 1 if suceeded or 0 if it fails
   @warning DmaPeripheralStructSetup() must be called first

**/

int DmaStructPtrOutSetup(int iChan, int iNumVals, const unsigned char *pucTX_DMA)
{
   int iByt = 0;  // number of bytes per dma value to be transmitted
   switch(dmaChanDesc[iChan].ctrlCfg.Bits.src_size)
   {
      case  SIZE_BYTE:
         iByt = 1;
         break;
      case SIZE_HWORD:
         iByt = 2;
         break;
      case SIZE_WORD:
         iByt = 4;
         break;
      default:
         return 0;
   }
   
   switch (iChan)
   {   
      case SPI0TX_C:  // SPI0 Tx 
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_SPI0->SPITX); 
		   break;
         
      case SPI1TX_C:  // SPI1 Tx 
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_SPI1->SPITX); 
		   break;
 			 
      case UARTTX_C:  // UART Tx
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_UART->COMTX); 
         break;
				 
		case I2C0STX_C:  // I2C0 Slave Tx
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_I2C0->I2CSTX); 
         break;
				 
      case I2C0M_C:  // I2C0 Master
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_I2C0->I2CMTX); 
         break;
      
		case I2C1STX_C:  // I2C1 Slave Tx
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_I2C1->I2CSTX); 
         break;
				 
      case I2C1M_C:  // I2C0 Master
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_I2C1->I2CMTX); 
         break;
      
      case Flash_C:  // Flash
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_FEE->FEEFLDATA1);
         dmaChanDesc[iChan].ctrlCfg.Bits.r_power      = 1;   //2 transactions per req
         break;

      case SPI0TX_C+ALTERNATE:  // SPI0 Tx Alternate
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_SPI0->SPITX); 
		   break;
         
      case SPI1TX_C+ALTERNATE:  // SPI1 Tx Alternate
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_SPI1->SPITX); 
		   break;
 			 
      case UARTTX_C+ALTERNATE:  // UART Tx Alternate
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_UART->COMTX); 
         break;
				 
		 case I2C0STX_C+ALTERNATE:  // I2C0 Slave Tx Alternate
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_I2C0->I2CSTX); 
         break;
				 
      case I2C0M_C+ALTERNATE:  // I2C0 Master Alternate
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_I2C0->I2CMTX); 
         break;
      
		 case I2C1STX_C+ALTERNATE:  // I2C1 Slave Tx Alternate
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_I2C1->I2CSTX); 
         break;
				 
      case I2C1M_C+ALTERNATE:  // I2C0 Master Alternate
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_I2C1->I2CMTX); 
         break;	      
	
      case Flash_C+ALTERNATE:  // Flash Alternate
         dmaChanDesc[iChan].srcEndPtr    = (unsigned int)(pucTX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].destEndPtr   = (unsigned int)(&pADI_FEE->FEEFLDATA1);
         dmaChanDesc[iChan].ctrlCfg.Bits.r_power      = 1;   //2 transactions per req

         break;
	 		 
	   default:	 
		     return 0;	
	   }
		 return 1;
}
/**
   @brief int DmaStructPtrInSetup(int iChan, int iNumVals, unsigned char *pucRX_DMA);
	 			==========For DMA operations where the source is fixed (peripheral register is fixed)
   @param iChan :{SPI0RX_C,SPI1RX_C,UARTRX_C,
         I2C0SRX_C,I2C0M_C,I2C1SRX_C,I2C1M_C,
         SPI0RX_C+ALTERNATE,SPI1RX_C+ALTERNATE,
         UARTRX_C+ALTERNATE,I2C0SRX_C+ALTERNATE,
         I2C0M_C+ALTERNATE,I2C1SRX_C+ALTERNATE,I2C1M_C+ALTERNATE}
    - 1 or SPI0RX_C   for the SPI0 Receive channel        - primary structure
    - 3 or SPI1RX_C   for the SPI1 Receive channel        - primary structure
    - 5 or UARTRX_C   for the UART Receive channel        - primary structure
    - 7 or I2C0SRX_C  for the I2C0 Slave receive channel  - primary structure 
    - 8 or I2C0M_C    for the I2C0 Master channel         - primary structure
    - 10 or I2C1SRX_C for the I2C1 Slave receive channel  - primary structure
    - 11 or I2C1M_C   for the I2C1 Master channel         - primary structure
    - 17 or SPI0RX_C+ALTERNATE   for the SPI0 Receive channel        - ALTERNATE structure
    - 19 or SPI1RX_C+ALTERNATE   for the SPI1 Receive channel        - ALTERNATE structure
    - 21 or UARTRX_C+ALTERNATE   for the UART Receive channel        - ALTERNATE structure 
    - 23 or I2C0SRX_C+ALTERNATE  for the I2C0 Slave receive channel  - ALTERNATE structure 
    - 24 or I2C0M_C+ALTERNATE    for the I2C0 Master channel         - ALTERNATE structure 
    - 26 or I2C1SRX_C+ALTERNATE  for the I2C1 Slave receive channel  - ALTERNATE structure 
    - 27 or I2C1M_C+ALTERNATE    for the I2C1 Master channel         - ALTERNATE structure 
   @param iNumVals :{1-1024}
    - 1 to 1024. Number of values to transfer
   @param *pucRX_DMA :{0-0xFFFFFFFF}
    - Pass pointer to destination address for DMA transfers		
   @return 1 if suceeded or 0 if it fails
   @warning DmaPeripheralStructSetup() must be called first

**/
int DmaStructPtrInSetup(int iChan, int iNumVals, unsigned char *pucRX_DMA)
{
   int iByt = 0;  // number of bytes per dma value to be transmitted
   switch(dmaChanDesc[iChan].ctrlCfg.Bits.src_size)
   {
      case  SIZE_BYTE:
         iByt = 1;
         break;
      case SIZE_HWORD:
         iByt = 2;
         break;
      case SIZE_WORD:
         iByt = 4;
         break;
      default:
         return 0;
   }
   
   switch (iChan)
   {   
      case SPI0RX_C:  // SPI0 Rx 
         dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].srcEndPtr   = (unsigned int)(&pADI_SPI0->SPIRX); 
		   break;
         
      case SPI1RX_C:  // SPI1 Rx 
         dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].srcEndPtr     = (unsigned int)(&pADI_SPI1->SPIRX); 
		   break;
			 
      case UARTRX_C:  // UART Rx 
         dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].srcEndPtr   = (unsigned int)(&pADI_UART->COMRX); 
         break;
			 
      case I2C0SRX_C:  // I2C0 Slave Rx 
         dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].srcEndPtr   = (unsigned int)(&pADI_I2C0->I2CSRX); 
         break;
			 
      case I2C0M_C:  // I2C0 Master Rx 
         dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].srcEndPtr   = (unsigned int)(&pADI_I2C0->I2CMRX); 
		   break;

      case I2C1SRX_C:  // I2C1 Slave Rx 
         dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].srcEndPtr   = (unsigned int)(&pADI_I2C1->I2CSRX); 
         break;
			 
      case I2C1M_C:  // I2C1 Master Rx 
         dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].srcEndPtr   = (unsigned int)(&pADI_I2C1->I2CMRX); 
		   break;
      
      case SPI0RX_C+ALTERNATE:  // SPI0 Rx Alternate
         dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].srcEndPtr   = (unsigned int)(&pADI_SPI0->SPIRX); 
		   break;
         
      case SPI1RX_C+ALTERNATE:  // SPI1 Rx Alternate
         dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].srcEndPtr     = (unsigned int)(&pADI_SPI1->SPIRX); 
		   break;
			 
      case UARTRX_C+ALTERNATE:  // UART Rx Alternate
         dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].srcEndPtr   = (unsigned int)(&pADI_UART->COMRX); 
         break;
			 
      case I2C0SRX_C+ALTERNATE:  // I2C0 Slave Rx Alternate
         dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].srcEndPtr   = (unsigned int)(&pADI_I2C0->I2CSRX); 
         break;
			 
      case I2C0M_C+ALTERNATE:  // I2C0 Master Rx Alternate
         dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].srcEndPtr   = (unsigned int)(&pADI_I2C0->I2CMRX); 
		   break;

      case I2C1SRX_C+ALTERNATE:  // I2C1 Slave Rx Alternate
         dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].srcEndPtr   = (unsigned int)(&pADI_I2C1->I2CSRX); 
         break;
			 
      case I2C1M_C+ALTERNATE:  // I2C1 Master Rx Alternate
         dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA+ iByt*(iNumVals - 0x1) );
         dmaChanDesc[iChan].srcEndPtr   = (unsigned int)(&pADI_I2C1->I2CMRX); 
		   break;

      
      default:	 
         return 0;	
      }
   return 1;
}
/**
   @brief int DmaGo(unsigned int iChan, int iNumx, int iCfg)		
            ==========Used to re-enable DMA config structure
   @param iChan :{SPI0TX_C,SPI0RX_C,SPI1TX_C,SPI1RX_C,UARTTX_C,UARTRX_C,
         I2C0STX_C,I2C0SRX_C,I2C0M_C,I2C1STX_C,I2C1SRX_C,I2C1M_C,ADC_C,Flash_C,
         SPI0TX_C+ALTERNATE,SPI0RX_C+ALTERNATE,SPI1TX_C+ALTERNATE,SPI1RX_C+ALTERNATE,
         UARTTX_C+ALTERNATE,UARTRX_C+ALTERNATE,I2C0STX_C+ALTERNATE,I2C0SRX_C+ALTERNATE,
         I2C0M_C+ALTERNATE,I2C1STX_C+ALTERNATE,I2C1SRX_C+ALTERNATE,I2C1M_C+ALTERNATE,
         ADC_C+ALTERNATE,Flash_C+ALTERNATE}
    - 0 or SPI0TX_C   for the SPI0 Transmit channel       - primary structure
    - 1 or SPI0RX_C   for the SPI0 Receive channel        - primary structure
    - 2 or SPI1TX_C   for the SPI1 Transmit channel       - primary structure
    - 3 or SPI1RX_C   for the SPI1 Receive channel        - primary structure
    - 4 or UARTTX_C   for the UART Transmit channel       - primary structure
    - 5 or UARTRX_C   for the UART Receive channel        - primary structure
    - 6 or I2C0STX_C  for the I2C0 Slave transmit channel - primary structure 
    - 7 or I2C0SRX_C  for the I2C0 Slave receive channel  - primary structure 
    - 8 or I2C0M_C    for the I2C0 Master channel         - primary structure
    - 9 or I2C1STX_C  for the I2C1 Slave transmit channel - primary structure 
    - 10 or I2C1SRX_C for the I2C1 Slave receive channel  - primary structure
    - 11 or I2C1M_C   for the I2C1 Master channel         - primary structure
    - 12 or ADC_C     for the ADC output channel          - primary structure 
    - 13 or Flash_C   for the Flash channel               - primary structure 
    - 16 or SPI0TX_C+ALTERNATE   for the SPI0 Transmit channel       - ALTERNATE structure
    - 17 or SPI0RX_C+ALTERNATE   for the SPI0 Receive channel        - ALTERNATE structure
    - 18 or SPI1TX_C+ALTERNATE   for the SPI1 Transmit channel       - ALTERNATE structure
    - 19 or SPI1RX_C+ALTERNATE   for the SPI1 Receive channel        - ALTERNATE structure
    - 20 or UARTTX_C+ALTERNATE   for the UART Transmit channel       - ALTERNATE structure 
    - 21 or UARTRX_C+ALTERNATE   for the UART Receive channel        - ALTERNATE structure 
    - 22 or I2C0STX_C+ALTERNATE  for the I2C0 Slave transmit channel - ALTERNATE structure 
    - 23 or I2C0SRX_C+ALTERNATE  for the I2C0 Slave receive channel  - ALTERNATE structure 
    - 24 or I2C0M_C+ALTERNATE    for the I2C0 Master channel         - ALTERNATE structure 
    - 25 or I2C1STX_C+ALTERNATE  for the I2C1 Slave transmit channel - ALTERNATE structure 
    - 26 or I2C1SRX_C+ALTERNATE  for the I2C1 Slave receive channel  - ALTERNATE structure 
    - 27 or I2C1M_C+ALTERNATE    for the I2C1 Master channel         - ALTERNATE structure 
    - 28 or ADC_C+ALTERNATE      for the ADC output channel          - ALTERNATE structure 
    - 29 or Flash_C+ALTERNATE    for the Flash channel               - ALTERNATE structure 
   @param iCfg :{DMA_STOP|DMA_BASIC|DMA_AUTO|DMA_PING|DMA_PSG_PRI|DMA_PSG_ALT}
    - Choose one of DMA_STOP,DMA_BASIC,DMA_AUTO,DMA_PING,DMA_PSG_PRI,DMA_PSG_ALT
   @param iNumx :{1-1024}
    - Number of values to transfer
 @return 1:

**/
int DmaGo(unsigned int iChan, int iNumx, int iCfg)
{
//  	iChan--;
	dmaChanDesc[iChan].ctrlCfg.Bits.cycle_ctrl       = (iCfg & 0x7);
	dmaChanDesc[iChan].ctrlCfg.Bits.n_minus_1        = iNumx - 0x1;
	return 1;
}

/**
   @brief int AdcDmaReadSetup(int iChan, int iNumVals, int iAin, unsigned int *pucRX_DMA)	
         ==========Sets up DMA config structure when moving ADC results direct to memory
   @param iChan :{ADC_C,ADC_C+ALTERNATE}
    - 12 or ADC_C     for the ADC output channel          - primary structure 
    - 26 or ADC_C+ALTERNATE      for the ADC output channel          - ALTERNATE structure    
  @param iNumVals :{1-1024}
    - 1 to 1024. Number of values to transfer
  @param iAin :{AIN0,AIN1,AIN2,AIN3,AIN4,AIN5,AIN6,AIN7,AIN8,AIN9,AIN_IDAC4,
               AIN_IDAC5,AIN_IDAC3,AIN_IDAC0,AIN_IDAC1,AIN_IDAC2,TEMP_SENSOR,VREFP_PADC,PVDD_GAIN,IOVDD_2,
               AVDD_2,VREFN_PADC}	
		- ADCCHA.[4:0]
   @param *pucRX_DMA :{0-0xFFFFFFFF}
    - Pass pointer to destination address for DMA transfers		
   @return 1 if suceeded or 0 if it fails
   @warning DmaPeripheralStructSetup() must be called first

**/
int AdcDmaReadSetup(int iChan, int iNumVals, int iAin, unsigned int *pucRX_DMA)
 {
   int iByt = 4;  // number of bytes per dma value to be transmitted

   switch (iChan)
   {   
       case ADC_C: // ADC write to control registers
          dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA) + iByt*(iNumVals - 0x1);
          dmaChanDesc[iChan].srcEndPtr   = (unsigned int)(&pADI_ADC->ADCDAT[iAin]); 
 		   break;
        case ADC_C+ALTERNATE:
          dmaChanDesc[iChan].destEndPtr    = (unsigned int)(pucRX_DMA) +  iByt*(iNumVals - 0x1);
          dmaChanDesc[iChan].srcEndPtr   = (unsigned int)(&pADI_ADC->ADCDAT[iAin]); 
 		   break; 
      default:	 
         return 0;	
      }
   return 1;	
 }


/**@}*/


