/**
 *****************************************************************************
   @addtogroup fee 
   @{

   @file     FeeLib.c
   @brief    Set of Flash peripheral functions.
   @version  V1.0
   
   @internal   001 @endinternal
   @author   ADI
   @date     April 2015
   @par Revision History:
   - V0.1, March 2013: initial version.
   - V0.2, October 2013: added FeeProTmp(), FeeWrPro(), FeeIntCfg(), FeeAuto(),
                        FeeClrCache(), FeeIntAbt() and FeeAbtAdr()
   - V0.3, July 2014: added FeeCacheClr() and FeeCacheEn().
   - V0.4, September 2014: added FeeCrc().
   - V1.0, April 2015: Changed library format

All files provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include "FeeLib.h"

/**
   @brief int FeeMErs(int iBlock);
      ========== Performs a mass erase if the flash controller is not busy.

   @param iBlock :{0,1}
      - 0 Mass erase block 0
      - 1 Mass erase block 1
   @return 1 if the command was issued, 0 if the the flash controller was busy.
**/

int FeeMErs(int iBlock)
{
   if((pADI_FEE->FEESTA&1)!=0) return 0;
   pADI_FEE->FEEKEY =  0xF123F456;
   if(iBlock) pADI_FEE->FEECMD = 0x6;
   else       pADI_FEE->FEECMD = 0x5;
   return 1;
}

/**
   @brief int FeePErs(unsigned long lAddr);
      ========== Performs a page erase if the flash controller is not busy.

   @param lAddr :{0-0x3FFFF}
      - Specifies an address in the page to be erased

   @return 1 if the command was issued, 0 if the the flash controller was busy.
**/

int FeePErs(unsigned long lAddr)
{
   if((pADI_FEE->FEESTA&1)!=0) return 0;

   pADI_FEE->FEEKEY =  0xF123F456;
   pADI_FEE->FEEADR0 = lAddr;
   pADI_FEE->FEECMD = 0x1;
   return 1;
}

/**
   @brief int FeeWr(unsigned long lAddr, unsigned long long udData);
      ========== Performs a 64 bit write if the flash controller is not busy.

   @param lAddr :{0-0x3FFFF}
      - Specifies the address to be written.
   @param udData :{0-0xFFFFFFFFFFFFFFFF}
      - Specifies the 64 bit data to be written.

   @return 1 if the command was issued, 0 if the the flash controller was busy.   
**/

int FeeWr(unsigned long lAddr, unsigned long long udData)
{
   if((pADI_FEE->FEESTA&1)!=0) return 0;

   pADI_FEE->FEEKEY =  0xF123F456;
   pADI_FEE->FEEFLADR = lAddr;
   pADI_FEE->FEEFLDATA0 = udData;
   pADI_FEE->FEEFLDATA1 = udData>>32;
   pADI_FEE->FEECMD = 0x4;
   return 1;
}

/**
   @brief int FeeSta(void);
      ========== Returns the status of the flash controller.

   @return value of FEESTA
      See User guide for bit descriptions.
**/

int FeeSta(void)
{
   return pADI_FEE->FEESTA;
}

/**
   @brief int FeeSign(unsigned long ulStartAddr, unsigned long ulEndAddr);
   ========== Perform flash signature check.

   @param ulStartAddr :{0-0x3FFFF}
      - An address in the first page to be checked;
   @param ulEndAddr :{0-0x3FFFF}
      - An address in the last page to be checked;

   @return 1 if the command was issued, 0 if the the flash controller was busy.
**/

int FeeSign(unsigned long ulStartAddr, unsigned long ulEndAddr)
{
   if((pADI_FEE->FEESTA&1)!=0) return 0;
   
   pADI_FEE->FEEADR0 = ulStartAddr;
   pADI_FEE->FEEADR1 = ulEndAddr;
   pADI_FEE->FEEKEY =  0xF123F456;
   
   pADI_FEE->FEECMD = 0x2;
   return 1;
}

/**
   @brief int FeeSig();
      ========== Return the flash signature calculated by the controller.

   @return FEESIG   
**/

int FeeSig(void)
{
   return pADI_FEE->FEESIG;
}
  
/**
   @brief int FeeProTmp(unsigned long long udKey, int iRdPro);
      ========== Temporarily enables write and/or read protection on the part.
            Write protection is disabled after a reset

   @param llKey :{0-0xFFFFFFFFFFFFFFFF}
      - Block 0 is write protected by clearing bit 0, Block 1 is write protected
      by clearing bit 1 and so forth for all 64 blocks
   @param iRdPro : {FEECON1_DBG_EN|FEECON1_DBG_DIS}
      - 1 or FEECON1_DBG_EN to enable JTAG
      - 0 or FEECON1_DBG_DIS to disable JTAG and read protect the part
   @return 1
   @note If you only want to read protect the part udKey should be 0xFFFFFFFFFFFFFFFF
   @warning If you read protect a part you will lose debug access
**/    

int FeeProTmp(unsigned long long udKey, int iRdPro)
{
   pADI_FEE->FEEPRO0 = (udKey & 0xFFFFFFFF);
   pADI_FEE->FEEPRO1 = udKey >> 32;
   pADI_FEE->FEEKEY = 0xF123F456;
   if (iRdPro)
      pADI_FEE->FEECON1 |= FEECON1_DBG_EN;
   else
      pADI_FEE->FEECON1 &= (~FEECON1_DBG_EN);
   pADI_FEE->FEEKEY = 0;
   
   return 1;
}

/**
   @brief int FeeWrPro(unsigned long lKey);
         ========== Enables write protection on the part.

   @param lKey :{0-0xFFFFFFFFFFFFFFFF}
      - Block 0 is write protected by clearing bit 0, Block 1 is write protected
      by clearing bit 1 and so forth for all 64 blocks
   @return 1
   @note This function should be used for debugging. If protection is required during
         production the protection patterns should be stored within the executable image
   
**/

int FeeWrPro(unsigned long long udKey)
{
   int i1;
   i1 = FeeWr(0x1FFF0, (udKey & 0xFFFFFFFF) | 0xFFFFFFFF00000000);
   if(!i1) 
      return 0;
   while(FeeSta()&1);
   i1 = FeeWr(0x3FFF0, (udKey>>32) | 0xFFFFFFFF00000000);
   if(!i1) 
      return 0;
   return 1;
}

/**
   @brief int FeeFAKey(unsigned long long udKey);
         ========== Writes the FA key to a specific location.

   @param lKey :{0-0xFFFFFFFFFFFFFFFF}
      - This key is used for failure analysis.
   @return 1
   @note This function should be used for debugging. During production the keys should be stored 
         within the executable image
   
**/

int FeeFAKey(unsigned long long udKey)
{
   int i1;
   i1 = FeeWr(0x1FFE8, (udKey & 0xFFFFFFFF) | 0xFFFFFFFF00000000);
   if(!i1) 
      return 0;
   while(FeeSta()&1);
   i1 = FeeWr(0x3FFE8, (udKey>>32) | 0xFFFFFFFF00000000);
   if(!i1) 
      return 0;
   return 1;
}

/**
   @brief int FeeIntCfg(int iIrq);
      ========== Enables/Disables Flash Interrupt sources.

   @param iIrq :{0|FEECON0_IENCMD|FEECON0_IWRALCOMP|FEECON0_IENERR}
      - 1 or FEECON0_IENCMD to enable command complete interrupts
      - 2 or FEECON0_IWRALCOMP to enable write almost complete interrupts
      - 4 or FEECON0_IENERR to enable command failed interrupts
   @return 1
   
**/   

int FeeIntCfg(int iIrq)
{
   pADI_FEE->FEECON0 = iIrq;
   return 1;
}

/**
   @brief int FeeAuto(int iCfg);
      ========== Configures the amount of automation that the flash
                  flash controller will perform for write operations.

   @param iCfg :{0,FEECON1_KHDMA,FEECON1_INCR}
      - 0 to manually write to flash registers for flash writes
      - 2 or FEECON1_KHDMA to enable DMA support in the flash controller
      - 4 or FEECON1_INCR to automatically increment the destination address
   @return 1
   
**/  

int FeeAuto(int iCfg)
{
   int i1 = pADI_FEE->FEECON1 & (~(FEECON1_KHDMA|FEECON1_INCR));
   pADI_FEE->FEEKEY = 0xF123F456;
   pADI_FEE->FEECON1 = i1 | iCfg;
   pADI_FEE->FEEKEY = 0;
   return 1;
}

/**
   @brief int FeeClrCache();
      ========== Clears enabled caches.

   @return 1
   @warning After the flash is modified if a reset isn't performed the
            cache should be cleared using this function
            This function is deprecated. Consider using FeeCacheClr().
**/  

int FeeClrCache(void)
{
   unsigned int uiCACHESETUP = pADI_FEE->CACHESETUP;
   if ((uiCACHESETUP & 0x2) == 0x2)
   {
      pADI_FEE->CACHEKEY = 0xf123f456;
      pADI_FEE->CACHESETUP |= 0x1;
      while ((pADI_FEE->CACHESTAT & 0x1) == 0x1)
      {  }
   }
   if ((uiCACHESETUP & 0x20000) == 0x20000)
   {
      pADI_FEE->CACHEKEY = 0xf123f456;
      pADI_FEE->CACHESETUP |= 0x10000;
      while ((pADI_FEE->CACHESTAT & 0x10000) == 0x10000)
      {  }
   }
   pADI_FEE->CACHEKEY = 0xf123f456;
   pADI_FEE->CACHESETUP = uiCACHESETUP;
   return 1;
}

/**
   @brief int FeeCacheClr(int iCache);
      ========== Clears caches selected by iCache.
   @param iCache :{CACHESETUP_IINIT_EN|CACHESETUP_DINIT_EN}
      - 1 or CACHESETUP_IEN_EN to clear instruction cache.
      - 0x10000 or CACHESETUP_DEN_EN to clear data cache.
   @return 1
   @warning Clears caches even if disabled.
**/  

int FeeCacheClr(int iCache)
   {
   unsigned int   ui1;
   
   ui1 = pADI_FEE->CACHESETUP;
   pADI_FEE->CACHEKEY = 0xf123f456;
   pADI_FEE->CACHESETUP = ui1|(iCache&(CACHESETUP_IINIT_EN|CACHESETUP_DINIT_EN));
   while(pADI_FEE->CACHESTAT&(iCache&(CACHESETUP_IINIT_EN|CACHESETUP_DINIT_EN)));
   pADI_FEE->CACHEKEY = 0xf123f456;
   pADI_FEE->CACHESETUP = ui1;
   return 1;
   }

/**
   @brief int FeeCacheEn(int iCache);
      ========== Switches selected caches on/off.
   @param iCache :{0|CACHESETUP_IEN_EN|CACHESETUP_DEN_EN}
      - 0 to switch caches off.
      - 2 or CACHESETUP_IEN_EN to switch on instruction cache.
      - 0x20000 or CACHESETUP_DEN_EN to switch on data cache.
   @return 1
**/  
int FeeCacheEn(int iCache)
   {
   pADI_FEE->CACHEKEY = 0xf123f456;
   pADI_FEE->CACHESETUP = (pADI_FEE->CACHESETUP
         &~(CACHESETUP_IEN_EN|CACHESETUP_DEN_EN))
         |(iCache&(CACHESETUP_IEN_EN|CACHESETUP_DEN_EN));
   return 1;
   }
   
/**
   @brief int FeeIntAbt(unsigned int iAEN0, unsigned int iAEN1);
         ========== Choose which interrupts can abort flash commands.

   @param iAEN0 :{}
   @param iAEN1 :{}
   @return 1
   
**/


int FeeIntAbt(unsigned int iAEN0, unsigned int iAEN1)
{
   pADI_FEE->FEEAEN0 = iAEN0;
   pADI_FEE->FEEAEN1 = iAEN1;
   return 1;
}

/**
   @brief int FeeAbtAdr();
         ========== Return the address of the location written when the write was aborted.

   @return pADI_FEE->FEEWRADDRA
   
**/

unsigned int FeeAbtAdr(void)
{
   return pADI_FEE->FEEWRADDRA;
}

/**
   @brief int FeeCrc(int iLen, int *aiData);
      ===== Calculate the CRC of iLen words from the location aiData point to. 

   @param iLen : { }
      The number of words to process.  
         This should be a multiple of 512 minus 1.
   @param aiData : { }
      The pointer to the first word to be processed.  
      This should point to an address that is a multiple of 512.
   @return The 24 bit CRC of the specified block.
   @warning Do not include pages from both flash blocks.
**/    
int FeeCrc(int iLen,int *aiData)
   {
   int i1,i2,iCrc;
   
   iCrc = 0xffffffff;            //Seed value.
   for(i1=0; i1<iLen; i1++)      //Starting at lowest address.
      {
      for(i2=31; i2>=0; i2--)    //MSB first.
         {
         iCrc <<= 1;             //Left shift.
         if((*(aiData+i1))&(1<<i2)) iCrc ^= 0x00800063;  //^= Polynomial.
         if(iCrc&(1<<24))           iCrc ^= 0x00800063;
         }
      }
   return(iCrc&0x00ffffff);      //Return 24 bits.
   }

/**@}*/
