/**
 *****************************************************************************
   @example  GPT.c
   @brief    Uses Timer 0 capture feature to capture time of external IRQ0 interrupt
             Timer 1 is configured to interrupt every 1.64mS
             Timer 2 interrupts every 105mS and the LED (P2.4) is toggled in the IRQ handler
             
   @version  V0.2
   @author   ADI
   @date     December 2015 
	 @par Revision History:
	 - V0.1, April 2013: initial version.
	 - V0.2, December 2015: Changed ADuCM3xx.h to DeviceHeader.h

              
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/


#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>


#include <WdtLib.h>
#include <GptLib.h>
#include <IntLib.h>
#include <DioLib.h>

int kickWDT = 1;
int cleared = 0;

void delay(long int);

int main (void)
{
   int i,j,k;
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);                                 // Disable the Watchdog timer resets
   
   DioOenPin(pADI_GP2,PIN4,1);      // Pin  2.4 configured as output
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }
   
   /* Configure Timer 0 */
   GptLd(pADI_TMR0,0xAAAA);                                                 // Load timer 0
   GptCfg(pADI_TMR0,TCON_CLK_HFXTAL,
   TCON_PRE_DIV16,TCON_MOD_FREERUN|TCON_ENABLE|TCON_EVENTEN);               // Enable Timer0 for free running mode using external 16Mhz crystal as clock source
   GptCapSrc(pADI_TMR0,0x100);                                              // Capture External IRQ0
   NVIC_EnableIRQ(TIMER0_IRQn);                                             // Enable Timer 0  interrupt source in NVIC
   
     /* Configure Timer 1 */
   GptLd(pADI_TMR1,0x8000);                                                 // Load timer 1 - 1.64mS period
   GptCfg(pADI_TMR1,TCON_CLK_HCLK,
   TCON_PRE_DIV1,TCON_MOD_PERIODIC|TCON_ENABLE);                            // Enable Timer1 for periodic mode using HCLK as clock source
   NVIC_EnableIRQ(TIMER1_IRQn);                                             // Enable Timer 1  interrupt source in NVIC
   
     /* Configure Timer 2 */
   GptLd(pADI_TMR2,0x8000);                                                 // Load timer 2 - 105mS
   GptCfg(pADI_TMR2,TCON_CLK_HCLK,
   TCON_PRE_DIV256,TCON_MOD_PERIODIC|TCON_ENABLE);                          // Enable Timer2 for periodic mode using HCLK as clock source
   NVIC_EnableIRQ(TIMER2_IRQn);                                             // Enable Timer 2  interrupt source in NVIC
   
    /* Configure External Interrupt 0 */

   EiCfg(EXTINT0, INT_EN, INT_FALL);                                        // Enable External IRQ 0 
   NVIC_EnableIRQ(EINT0_IRQn);                                              // Enable External IRQ0 interrupt source in NVIC

	DioOenPin(pADI_GP2,PIN4,1);                                              // Configure P2.4 as an output - to toggle LED with Timer 2
   DioSet(pADI_GP2, BIT4);

	DioOenPin(pADI_GP2,PIN0,1);                                              // Configure P2.0 as an output - to toggle with Timer 0
   DioSet(pADI_GP2, BIT0);
   
   DioOenPin(pADI_GP2,PIN1,1);                                              // Configure P2.1 as an output - to toggle with Timer 1
   DioSet(pADI_GP2, BIT1);
   while (1)
   {
  
	}
}
void GP_Tmr0_Int_Handler()
{
   volatile int uiT0STA = 0;
   
   uiT0STA = GptSta(pADI_TMR0);
   if ((uiT0STA&0x2)==0x2)
   {
      GptClrInt(pADI_TMR0,TSTA_CAP);
   }
   pADI_TMR0->TCLRI = 0x1; // Clear timer interrupt
   DioTgl(pADI_GP2, BIT0);                                                // Toggle  P2.0
}
void GP_Tmr1_Int_Handler()
{
   GptClrInt(pADI_TMR1,TSTA_TMOUT);                                        // Clear timer interrupt
    DioTgl(pADI_GP2, BIT1);                                                // Toggle P2.1
}
void GP_Tmr2_Int_Handler()
{
   volatile int uiT2STA = 0;
   
   uiT2STA = GptSta(pADI_TMR2);
   GptClrInt(pADI_TMR2,TSTA_TMOUT);                                         // Clear timer interrupt
   DioTgl(pADI_GP2, BIT4);                                                  // Toggle LED, P2.4
}

void Ext_Int0_Handler()
{
  
   pADI_GP2->GPTGL = 0x10;        // Toggle P2.4
   pADI_INTERRUPT->EICLR = 0x1;   // Clear the interrupt0 source
   
}



// Simple Delay routine
void delay (long int length)
{
	while (length >0)
    	length--;
}




