/** @example ResetMaintain.c
    @brief   This example shows the ability of GPIOs and of
             VDACs to maintain state after a software
             reset. By default this feature is enabled.
             When you press the IRQ0 button the VDAC0 is enabled
             and P0.0 and P0.1 are set to a known state and then a software
             reset is triggered
             

    @version V0.2
    @author  ADI
    @date    December 2015
		@par Revision History:
		- V0.1, April 2013: initial version.
		- V0.2, December 2015: Changed ADuCM3xx.h to DeviceHeader.h
 *****************************************************************************


              
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/


#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>
#include <WdtLib.h>
#include <IntLib.h>
#include <DioLib.h>
#include <UrtLib.h>
#include <DacLib.h>

void UartSetup(void);                             // UART Setup function
void TestButtonPress(void);                       // Sends string to UART when IRQ0 asserted
volatile unsigned char ucCOMSTA0 = 0;             // Variable used to store COMSTA0, UART status register
volatile unsigned char ucCOMIID0 = 0;             // Variable used to store COMIID0, UART Interrupt status register	 
volatile unsigned char ucComRx = 0;               // Variable used to read UART Rx buffer contents into 
unsigned char ucTxBufferEmpty  = 0;		           // Used to indicate that the UART Tx buffer is empty
unsigned char szTemp[64] = "";		              // Used to store ADC1 result before printing to UART
unsigned char ucButtonPress = 0;                  // used to detect IRQ0 interrupt

//#define RESET_MAINTAIN_DISABLE 1                 // Uncomment this if you want to see the GPIOs and 
                                                   // Analog outputs reset their state after a software reset
int main (void)
{
   int i,j,k,reset = 0;
   unsigned char nLen = 0; 
   
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);        // Disable the Watchdog timer resets
   
   DioOenPin(pADI_GP2,PIN4,1);      // Pin  2.4 configured as output
   //delay before program starts in
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }
   
   reset = pADI_RESET->RSTSTA;                    //Read the reset status
   pADI_RESET->RSTSTA = reset;                    //clear the reset status by writing the same value back to the MMR
   
   UartSetup();                                   // Call function to initialize UART
    /* Configure External Interrupt 0 */

   EiCfg(EXTINT0, INT_EN, INT_FALL);              // Enable External IRQ 0 
   NVIC_EnableIRQ(EINT0_IRQn);                    // Enable External IRQ0 interrupt source in NVIC
   sprintf ( (char*)szTemp, "ADuCM320i RESET 0x%X \r\n", reset ); // Scan string 
   nLen = strlen((char*)szTemp);                  // Call function to calcualte the length of scanned string
   if (nLen <64)
   {
      for ( i = 0 ; i < nLen ; i++ )	          // loop to send String to UART
       {
          ucTxBufferEmpty = 0;	                  // Clear flag
          UrtTx(pADI_UART,szTemp[i]);             // Load UART Tx register.
          while (ucTxBufferEmpty == 0)            // Wait for UART Tx interrupt
          {
          }
       }
     }
 
   DioSet(pADI_GP2, BIT4);
   while (1)
   {
      if (ucButtonPress == 1)
      {
         ucButtonPress = 0;
         //Set P0.0 and P1.1 to HIGH and LOW respectively
         DioOenPin(pADI_GP0,PIN0,1);
         DioOenPin(pADI_GP0,PIN1,1);
         DioSet(pADI_GP0, BIT0);
         DioClr(pADI_GP0, BIT1);
         
         DacPwrDwn(pADI_VDAC0,0);                             // power up VDAC0
         DacCfg(pADI_VDAC0,DAC0CON_MODE_12BIT,INT_REF);       // Turn on VDAC0, and use internal reference
         DacWr(pADI_VDAC0,0x07FF0000);    
    
         
#ifdef RESET_MAINTAIN_DISABLE
         pADI_RESET->RSTKEY =  0x2009;
         pADI_RESET->RSTKEY =  0x0426;
         pADI_RESET->RSTCFG = RSTCFG_GPIO_PLA_RETAIN_DIS;      //disable GPIO/PLA maintain state after software reset
         pADI_LV_RST->LVRST = LVRST_RETAIN_DIS;                //disable VDAC maintain state after software reset
#endif
         for (i = 0; i<100; i++)    //delay
         {
            DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
            for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
         }

         NVIC_SystemReset();        //software reset
        }
   }
}
// Function used to setup the UART block
void UartSetup(void)
{
   DioCfg(pADI_GP1,0x5);                          // Configure P1[1:0] as UART pins
   UrtCfg(pADI_UART,B9600,COMLCR_WLS_EIGHTBITS,0);// Configure UART for 9600 baud rate
  // UrtCfg(pADI_UART,B230400,COMLCR_WLS_EIGHTBITS,0);// Configure UART for 230400 baud rate
   UrtIntCfg(pADI_UART,COMIEN_ERBFI|
   COMIEN_ETBEI|COMIEN_ELSI);                     // Enable Rx, Tx and Rx buffer full Interrupts
   pADI_UART->COMLCR = 0x3;	
   NVIC_EnableIRQ(UART_IRQn);                     // Enable UART interrupt source in NVIC
}

void UART_Int_Handler()
{  	
      ucCOMSTA0 = UrtLinSta(pADI_UART);
      ucCOMIID0 = UrtIntSta(pADI_UART);
      if ((ucCOMIID0 & 0x2) == 0x2)	          // Transmit buffer empty
       {
	  ucTxBufferEmpty = 1;
       }	
      if ((ucCOMIID0 & 0x4) == 0x4)	          // Receive byte
       {
	 ucComRx	= UrtRx(pADI_UART);
       }
}
void Ext_Int0_Handler()
{
  
   pADI_GP2->GPTGL = 0x10;                        // Toggle P2.4
   pADI_INTERRUPT->EICLR = 0x1;                   // Clear the interrupt0 source
   ucButtonPress = 1;
}


// Simple Delay routine
void delay (long int length)
{
	while (length >0)
    	length--;
}




