/**
 *****************************************************************************
   @file     DmaLib.h
   @brief   Set of DMA peripheral functions.
   - DmaBase(), DmaSet(), DmaClr(), DmaSta() and DmaErr() apply to all DMA channels together.
   
   @internal   001 @endinternal
   @version    V1.0
   @author     ADI
   @date       April 2015
   @par Revision History:
   - V0.1, July 2013: initial version.
   - V0.2, September 2013: DmaStructPtrOutSetup() now needs a const pointer,
                           header modified for C++ compatibility
   - V0.3, November 2014: Removed undefined functions and corrected declaration
                          of AdcDmaReadSetup()
   - V1.0, April 2015: Release

All files provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.
**/
#ifdef __cplusplus
extern "C" {
#endif 

#include "DeviceHeader.h"

typedef struct
{
   unsigned int cycle_ctrl    :3;
   unsigned int next_useburst :1;
   unsigned int n_minus_1     :10;
   unsigned int r_power       :4;
   unsigned int src_prot_ctrl :3;
   unsigned int dst_prot_ctrl :3;
   unsigned int src_size      :2;
   unsigned int src_inc       :2;
   unsigned int dst_size      :2;
   unsigned int dst_inc       :2;
} CtrlCfgBits;

// Define the structure of a DMA descriptor.
typedef struct dmaDesc
{
   unsigned int srcEndPtr;
   unsigned int destEndPtr;
   union
   {
      unsigned int   ctrlCfgVal;
      CtrlCfgBits    Bits;
   } ctrlCfg ;
   unsigned int reserved4Bytes;
} DmaDesc;
//typedef enum {false = 0, true = !false} boolean;
// Suitable aLignment for the DMA descriptors
#define DMACHAN_DSC_ALIGN 0x200
/* CCD allocation must be an integral power of two, i.e., 12 channels is allocated as 16 */
#define CCD_SIZE                       (16)

extern int DmaBase(void);
extern DmaDesc * Dma_GetDescriptor(unsigned int iChan);
extern int DmaSet(int iMask, int iEnable, int iAlt, int iPriority);
extern int DmaClr(int iMask, int iDisable, int iAlt, int iPriority);
extern int DmaSta(void);
extern int DmaErr(int iErrClr);

extern int AdcDmaReadSetup(int iChan, int iNumVals, int iAin, unsigned int *pucRX_DMA);
extern int DmaPeripheralStructSetup(int iChan, int iCfg);
extern int DmaStructPtrOutSetup(int iChan, int iNumVals, const unsigned char *pucTX_DMA);
extern int DmaStructPtrInSetup(int iChan, int iNumVals, unsigned char *pucRX_DMA);
extern int DmaGo(unsigned int iChan, int iNumx, int iCfg);
//DMA channel numbers.
#define	SPI0TX_C	   0
#define	SPI0RX_C	   1
#define	SPI1TX_C	   2
#define	SPI1RX_C	   3
#define	UARTTX_C	   4
#define	UARTRX_C	   5
#define	I2C0STX_C	6
#define	I2C0SRX_C	7
#define	I2C0M_C	   8
#define	I2C1STX_C	9
#define	I2C1SRX_C	10
#define	I2C1M_C	   11
#define	ADC_C	      12
#define	Flash_C	   13

//DMA channel bit value.
#define	NONE		   0
#define	SPI0TX_B	   0x1
#define	SPI0RX_B	   0x2
#define	SPI1TX_B	   0x4
#define	SPI1RX_B	   0x8
#define	UARTTX_B	   0x10
#define	UARTRX_B	   0x20
#define	I2C0STX_B	0x40
#define	I2C0SRX_B	0x80
#define	I2C0M_B	   0x100
#define	I2C1STX_B	0x200
#define	I2C1SRX_B	0x400
#define	I2C1M_B	   0x800
#define	ADC_B	      0x1000
#define	Flash_B	   0x2000

#define ADC0DMAWRITE 0
#define ADC0DMAREAD  1
#define ADC1DMAWRITE 2
#define ADC1DMAREAD  3
#define SINC2DMAREAD 4

//DMA error bit.
#define	DMA_ERR_RD		0
#define	DMA_ERR_CLR		1

//DMA type (Cycl_ctrl)
#define	DMA_STOP		0
#define	DMA_BASIC		1
#define	DMA_AUTO		2
#define	DMA_PING		3
#define	DMA_MSG_PRI		4
#define	DMA_MSG_ALT		5
#define	DMA_PSG_PRI		6
#define	DMA_PSG_ALT		7

//Bytes per transfer.
#define	DMA_SIZE_BYTE		 0
#define	DMA_SIZE_HWORD	 0x1000000
#define	DMA_SIZE_WORD		 0x2000000


//Destination Increment per transfer.
#define	DMA_DSTINC_BYTE		0
#define	DMA_DSTINC_HWORD	0x40000000
#define	DMA_DSTINC_WORD		0x80000000
#define	DMA_DSTINC_NO		  0xC0000000

//Source Increment per transfer.
#define	DMA_SRCINC_BYTE		0
#define	DMA_SRCINC_HWORD	0x4000000
#define	DMA_SRCINC_WORD		0x8000000
#define	DMA_SRCINC_NO		  0xC000000

#define iPrimary        0
#define ALTERNATE      CCD_SIZE

#define SIZE_BYTE    0
#define SIZE_HWORD   1
#define SIZE_WORD    2

#ifdef __cplusplus
}
#endif
