/**
 *****************************************************************************  
   @addtogroup Idac 
   @{
   @file       IDacLib.c
   @brief      Set of IDAC peripheral functions.

   @internal   001 @endinternal
   @version    V1.0
	@author     ADI
	@date       April 2015
   @par Revision History:
   - V1.0, April 2015: Full release

All files provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.
Example sequence to setup IDAC1
 - IDacCfg(pADI_IDAC1,BW153Hz,0,IDAC0CON_CLRB);
 - IDacWr(pADI_IDAC1,0x8000000,0)
 

**/

#include	"IDacLib.h"

/**
	@brief int IDacWr(ADI_IDAC_TypeDef *pPort, int iData, int iSync)
			==========Writes the IDAC value.
	@param pPort :{pADI_IDAC0,pADI_IDAC1,pADI_IDAC2,pADI_IDAC3,pADI_IDAC4,pADI_IDAC5,pADI_IDAC6}
      - pADI_IDAC0 for IDAC0.
      - pADI_IDAC1 for IDAC1.
      - pADI_IDAC2 for IDAC2.
      - pADI_IDAC3 for IDAC3.
      - pADI_IDAC4 for IDAC4.
      - pADI_IDAC5 for IDAC5.
      - pADI_IDAC6 for IDAC6.
	@param iData :{}	\n
		- IDACxDAT
		- Data to output to IDAC.
   @param iSync :{0|IDAC0|IDAC1|IDAC2|IDAC3|IDAC4|IDAC5|IDAC6}	\n
		Set to set sync bit for associated IDAC channel
      - 0 disables synch function - IDAC channel updated immediately
      - 1 or IDAC0 to prevent IDAC0 channel data update until this bit is cleared
      - 2 or IDAC1 to prevent IDAC1 channel data update until this bit is cleared
      - 4 or IDAC2 to prevent IDAC2 channel data update until this bit is cleared
      - 8 or IDAC3 to prevent IDAC3 channel data update until this bit is cleared
      - 0x10 or IDAC4 to prevent IDAC4 channel data update until this bit is cleared
      - 0x20 or IDAC5 to prevent IDAC5 channel data update until this bit is cleared
      - 0x40 or IDAC6 to prevent IDAC6 channel data update until this bit is cleared
	@return DAC data.
**/
int IDacWr(ADI_IDAC_TypeDef *pPort, int iData, int iSync)
{
   pPort->IDACDAT = iData + iSync;
	return  pPort->IDACDAT;
}

/**
	@brief int IDacCfg(ADI_IDAC_TypeDef *pPort, int iBwidth, int iIsink, int iEn)
			==========Sets the output range of a DAC.
	@param pPort :{pADI_IDAC0,pADI_IDAC1,pADI_IDAC2,pADI_IDAC3,pADI_IDAC4,pADI_IDAC5,pADI_IDAC6}
      - pADI_IDAC0 for IDAC0.
      - pADI_IDAC1 for IDAC1.
      - pADI_IDAC2 for IDAC2.
      - pADI_IDAC3 for IDAC3.
      - pADI_IDAC4 for IDAC4.
      - pADI_IDAC5 for IDAC5.
      - pADI_IDAC6 for IDAC6.
	@param iBwidth :{BW262Khz|BW2_8Khz|BW1_4Khz|BW715Hz|BW357Hz|BW153Hz}
      - IDACxCON[5:2]
		- BW262Khz or 0 to set output bandwidth to 262kHz
      - BW2_8Khz or 5 to set output bandwidth to 2.8kHz
      - BW1_4Khz or 6 to set output bandwidth to 1.4kHz
      - BW715Hz or 7 to set output bandwidth to 715Hz
      - BW357Hz or 8 to set output bandwidth to 357Hz
      - BW153Hz or 9 to set output bandwidth to 153Hz
	@param iIsink :{0, IDAC0CON_PUL}
		- IDACxCON[1]
		- 0 to disable current sink
		- 2 or IDAC0CON_PUL to enable pull-sown current sink.
	@param iEn :{0|IDAC0CON_SHT_EN|IDAC0CON_CLRB}
		- IDACxCON.7,6
      - 0 to disable writes to IDAC output 
		- 0x80 or IDAC0CON_CLRB to enable writes to IDACxDAT register
		- 0x40 or IDAC0CON_SHT_EN to enable thermal shutdown of the IDAC output.
	@return new IDACCON.
**/
	
int IDacCfg(ADI_IDAC_TypeDef *pPort, int iBwidth, int iIsink, int iEn)
{
   pPort->IDACCON = iBwidth + iIsink + iEn;
	return  pPort->IDACCON; 
}

 /**
	@brief int IDacPwrDwn(ADI_IDAC_TypeDef *pPort, int iCfg)
			==========Powers up or down selected IDAC
        Need to call IDacCfg() after this function
		@param pPort :{pADI_IDAC0,pADI_IDAC1,pADI_IDAC2,pADI_IDAC3,pADI_IDAC4,pADI_IDAC5,pADI_IDAC6}
      - pADI_IDAC0 for IDAC0.
      - pADI_IDAC1 for IDAC1.
      - pADI_IDAC2 for IDAC2.
      - pADI_IDAC3 for IDAC3.
      - pADI_IDAC4 for IDAC4.
      - pADI_IDAC5 for IDAC5.
      - pADI_IDAC6 for IDAC6.
	@param iCfg :{IDAC0CON_PD_ON,IDAC0CON_PD_OFF}	\n
		- 0 or IDAC0CON_PD_ON to power up selected IDAC channel
		- 1 or IDAC0CON_PD_OFF to power down selected IDAC channel
	@return new IDACCON.
**/  
   
int IDacPwrDwn(ADI_IDAC_TypeDef *pPort, int iCfg)
{
   pPort->IDACCON = iCfg;
	return  pPort->IDACCON; 
}


/**@}*/
