/**
 *****************************************************************************
   @addtogroup mdio 
   @{
   @file       MdLib.c
   @brief      Set of MDIO peripheral functions.

   @internal   001 @endinternal
   @version    V1.0
   @author     ADI
   @date			December 2015
   @par Revision History:
   - V0.0, November 2011: initial version.
   - V0.1, July 2014: added MdInit().
   - V1.0, December 2015: Changed library format


All files provided by ADI, including this file, are
provided as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include "MdLib.h"

/**
	@brief int MdInit(int iPhySz, int iRst, int iDrvPp)	
		==========Optionally resets then sets PHYADR size and MDIO pin mode.
	@param iPhySz :{3,5}
		- 3 to set PHYADR size to 3.
		- 5 to set PHYADR size to default of 5.
	@param iRst :{0,1}
		- 0 does not cause a reset.
		- 1 resets the MDIO hardware. Bit clears automatically.
	@param iDrvPp :{0,1}
		- 0 Sets pin to default of open drain.
		- 1 Sets MDIO as push-pull.
	@return 1.
**/

int MdInit(int iPhySz, int iRst, int iDrvPp)
	{
    MDCON_MD_RST_BBA = (iRst!=0);
	MDCON_MD_DRV_BBA = (iDrvPp!=0);
	MDCON_MD_PHM_BBA = (iPhySz==3);
	return 1;
	}

/**
	@brief int MdOn(int iOn, int iRst, int iDrvPp)	
		==========Switches MDIO off/on in selected mode.
	@param iOn :{0,1}
		-0 to switch MDIO off.
		-1 to switch MDIO on.
	@param iRst :{0,1}
		- 0 does not cause a reset.
		- 1 resets the MDIO hardware.
	@param iDrvPp :{0,1}
		- 0 Sets pin to default of open drain.
		- 1 Sets MDIO as push-pull.
	@return iOn.
	@warning Use for revision S1 silicon only. Else use MdInit().
**/

int MdOn(int iOn, int iRst, int iDrvPp)
	{
/*    MDCON_MD_DIS_BBA = (iOn==0);  Not useable due to bitbanding issue
	MDCON_MD_RST_BBA = (iRst!=0);    identified by Bruce Luo.
	MDCON_MD_DRV_BBA = (iDrvPp!=0);     */
    pADI_MDIO->MDCON = (iRst!=0)|(pADI_MDIO->MDCON&0xFFFE);   
    pADI_MDIO->MDCON = (iOn<<1)|(pADI_MDIO->MDCON&0xFFFC) ;
    pADI_MDIO->MDCON = (iDrvPp<<2)|(pADI_MDIO->MDCON&0xFFFA);
	return (int)!MDCON_MD_DIS_BBA;
	}

/**
	@brief int MdCfg(int iPhyAdr, int iPASel, int iDevAdd)	
			==========Configures MDIO for communications.
	@param iPhyAdr :{0-31}
		- Software supplied PHYADR bits.
	@param iPASel :{0-31}
		- For each of the 5 bits:
		- 0 selects corresponding bit from PRTADR pins for PHYADR.
		- 1 selects corresponding bit from iPhyAdr for PHYADR.
	@param iDevAdd :{0-31}
		- DEVADD that the device will respond to.  
	@return The PHYADR that the device will respond to.
	@warning Changing PRTADR pins will change PHYADR.
		    For CFP only 1 is valid for DEVADD.
**/

int MdCfg(int iPhyAdr, int iPASel, int iDevAdd)
	{
	pADI_MDIO->MDPHY = iPhyAdr|(iPASel<<5)|(iDevAdd<<10);
	return (pADI_MDIO->MDPIN&~iPASel)|(iPhyAdr&iPASel);
	}

/**
	@brief int MdRxd(void)	
			==========Reads data supplied by MDIO frame.
	@return The MDRXD value from the last MDIO data frame.
	@warning None
**/

int MdRxd(void)
	{
	return pADI_MDIO->MDRXD;
	}

/**
	@brief int MdTxd(int iTxDat)	
			==========Writes data to transmit by MDIO frame.
	@param iTxDat :{0-65535}
		- Data to send on MDIO.
	@return iTxDat.
	@warning None
**/

int MdTxd(int iTxDat)
	{
	pADI_MDIO->MDTXD = iTxDat;
	return iTxDat;
	}

/**
	@brief int MdAdr(void)	
			==========Reads address supplied by MDIO frame.
	@return The address supplied by the most recent MDIO address frame.
	@warning None.
**/

int MdAdr(void)
	{
	return pADI_MDIO->MDADR;
	}

/**
	@brief int MdPin(void)	
			==========Reads the PRTADR pins.
	@return The PRTADR pins.
	@warning None
**/

int MdPin(void)
	{
	return pADI_MDIO->MDPIN;
	}

/**
	@brief int MdSta(void)	
			==========Reads the MDIO frame status.
	@return Frame status bits.
		- Bit0 Set at end of Write frame.
		- Bit1 Set at end of Address frame.
		- Bit2 Set at end of PostReadIncAdd frame.
		- Bit3 Set at end of Read frame.
		- Bit4 Set at end of matching PHYADR.
		- Bit5 Set at end of non-matching PHYADR.
		- Bit6 Set at end of DEVADD if PHYADR and DEVADD matching.
		- Bit7 Set at end of DEVADD if PHYADR or DEVADD non-matching.
	@warning All status bit are reset by this function.
**/

int MdSta(void)
	{
	return pADI_MDIO->MDSTA;
	}

/**
	@brief int MdRdI(void)	
			==========Reads interupt enable bits.
	@return Interrupt enable bits.
		- Bit0 Enables Write frame interrupt.
		- Bit1 Enables Address interrupt.
		- Bit2 Enables PostReadIncAdd interrupt.
		- Bit3 Enables Read interrupt.
		- Bit4 Enables matching PHYADR interrupt.
		- Bit5 Enables non-matching PHYADR interrupt.
		- Bit6 Enables PHYADR and DEVADD matching interrupt.
		- Bit7 Enables PHYADR or DEVADD non-matching interrupt.
	@warning None.
**/

int MdRdI(void)
	{
	return pADI_MDIO->MDIEN;
	}

/**
	@brief int MdIen(int iMdIEn)	
			==========Writes interupt enable bits.
	@param iMdIEn :{0|MDIEN_MD_WRFI|MDIEN_MD_ADRI|MDIEN_MD_INCFI|MDIEN_MD_RDFI
                   |MDIEN_MD_PHYMI|MDIEN_MD_PHYNI|MDIEN_MD_DEVMI|MDIEN_MD_DEVNI}
		- Set bits to enable sources as listed below:
        - 0 for no interrupt.
		- 1 or MDIEN_MD_WRFI Enables Write frame interrupt.
		- 2 or MDIEN_MD_ADRI Enables Address interrupt.
		- 4 or MDIEN_MD_INCFI Enables PostReadIncAdd interrupt.
		- 8 or MDIEN_MD_RDFI Enables Read interrupt.
		- 0x10 or MDIEN_MD_PHYMI Enables matching PHYADR interrupt.
		- 0x20 or MDIEN_MD_PHYNI Enables non-matching PHYADR interrupt.
		- 0x40 or MDIEN_MD_DEVMI Enables PHYADR and DEVADD matching interrupt.
		- 0x80 or MDIEN_MD_DEVNI Enables PHYADR or DEVADD non-matching interrupt.
	@return MDIEN.
	@warning None.
**/

int MdIen(int iMdIEn)
	{
	pADI_MDIO->MDIEN = iMdIEn;
	return pADI_MDIO->MDIEN;
	}

/**
	@brief int MdFrm(void)	
			==========Gets details of last frame received.
	@return Received frame details as follows:
		- Bit0 and 1 Give OP of frame.
            - 0 or MD_OP_ADF = Address frame
            - 1 or MD_OP_WRF = Write frame
            - 2 or MD_OP_RDF = Read frame
            - 3 or MD_OP_INCF = PostReadIncAdd frame
		- Bits 2 to 6 give received PHYADR.
		- Bit7 to 11 give received DEVADD.
	@warning None.
**/

int MdFrm(void)
	{
	return pADI_MDIO->MDFRM;
	}

/**@}*/
