/**
 *****************************************************************************
   @example  FlashPingPongDma.c
   @brief    Flash DMA example
      This projects shows how to use the ping pong DMA to store a large array of data
      from RAM into flash

   @version  V0.1
   @author   ADI
   @date     December 2015

   
All files for the device are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.


**/

#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>
#include <IntLib.h>
#include <WdtLib.h>
#include <DioLib.h>
#include <DmaLib.h>


#define DATA_SZ (64)
void delay(long int);
void SPI1INIT(void);
void DMAINIT(void);


unsigned int done_cnt = 0;
int dma_done = 0;

unsigned char ucIrqCnt = 0;                           // Used to count number of flash DMA interrupts
unsigned char ucWriteData = 0;                        // Flag used to setup 2nd and subsequent transfers
unsigned int uiFlashData0[DATA_SZ];
unsigned int uiFlashData1[DATA_SZ];
unsigned long ulFEESTA;
unsigned int uiAdr = 0x21000;
unsigned int numDmaTran = DATA_SZ;

int main (void)
{
   
   int i,j,k;
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);                                   // Disable the Watchdog timer resets
   
   DioOenPin(pADI_GP2,PIN4,1);                                                // Pin  2.4 configured as output
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }

   for(i=0; i<DATA_SZ; i++)
   {
      uiFlashData0[i] = 0x12345600+i;
      uiFlashData1[i] = 0x9ABCDF00+i;
   }
   
   DMAINIT();                                                                 // Initialize DMA controller
   
   EiCfg(EXTINT0,INT_EN,INT_RISE);                                            // Enable EINT0 - used to triger transfer
   NVIC_EnableIRQ(EINT0_IRQn);                                                // Enable IRQ0 interrupt
   NVIC_EnableIRQ(DMA_FLASH_IRQn);                                            // Flash DMA interrupt enable
   
   DmaPeripheralStructSetup(Flash_C,DMA_DSTINC_NO|
      DMA_SRCINC_WORD|DMA_SIZE_WORD);                                         // Setup primary Flash DMA structure  
   DmaPeripheralStructSetup(Flash_C+ALTERNATE,DMA_DSTINC_NO|
      DMA_SRCINC_WORD|DMA_SIZE_WORD);                                         // Setup alternate Flash DMA structure  
   
   DmaStructPtrOutSetup(Flash_C,numDmaTran,(unsigned char*)uiFlashData0);              // Setup source/destination pointer fields
   DmaStructPtrOutSetup(Flash_C+ALTERNATE,numDmaTran,(unsigned char*)uiFlashData1);    // Setup source/destination pointer fields
   
   DmaSet(0,Flash_B,0,0);                                               // Enable DMA structure for flash
   DmaGo(Flash_C,numDmaTran,DMA_PING);                                  // Enable primary Flash DMA structure
   DmaGo(Flash_C+ALTERNATE,numDmaTran,DMA_PING);                        // Enable alternate Flash DMA structure
   pADI_FEE->FEEFLADR = uiAdr;
   pADI_FEE->FEEKEY = 0xF123F456;
   pADI_FEE->FEECON1 |= (FEECON1_KHDMA_EN);                           // Enable Flash DMA mode, this starts the DMA transfer
   
   while (1)
   {
      delay(0x60000);         // Delay routine
   }
}


void DMAINIT(void)  
{
   DmaBase();                 
}
// Simple Delay routine
void delay (long int length)
{
   while (length >0)
      length--;
}
void WakeUp_Int_Handler(void)
{
  
}
void Ext_Int2_Handler ()
{           

}
void Ext_Int0_Handler ()
{         
   EiClr(EXTINT0);                                          // Clear EINT0 interrupt flag
   ucWriteData = 1;
}

void UART_Int_Handler ()
{

} 
void DMA_Flsh_Int_Handler()
{
   done_cnt ++;
   if(done_cnt < 10)                //do only 10 DMA transactions
   {
      if(done_cnt %2)               //if it's the odd interrupt resetup the primary channel
      {
         DmaStructPtrOutSetup(Flash_C,numDmaTran,(unsigned char*)uiFlashData0);
         DmaGo(Flash_C,numDmaTran,DMA_PING);
      }
      else                          //if it's the even interrupt resetup the alternate channel
      {
         DmaStructPtrOutSetup(Flash_C+ALTERNATE,numDmaTran,(unsigned char*)uiFlashData1);
         DmaGo(Flash_C+ALTERNATE,numDmaTran,DMA_PING);
      }
   }
   else
   {
      pADI_FEE->FEEKEY = 0xF123F456;
      pADI_FEE->FEECON1 &= (~FEECON1_KHDMA_EN);           // Disable Flash DMA mode
      dma_done = 1;
   }
   
}



