/**
 *****************************************************************************
   @example  VDAC_Demo.c
   @brief    VDAC test file
  Decription: Demo for VDACs
      - Enables IRQ0 - pushbutton IRQx 
      - After each button press, VDAC outputs are updated
   @version  V0.3
   @author   ADI
   @date     December 2015
   @par Revision History:
   - V0.1, June 2013: initial version. 
   - V0.2, December 2013: Turning off maintain GPIO/PLA/LV die after software reset
	 - V0.3, December 2015: Changed ADuCM3xx.h to DeviceHeader.h

All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>

#include <IntLib.h>
#include <DacLib.h>
#include <WdtLib.h>
#include <DioLib.h>

// Function Definitions

void delay(long int);
void VDACSetup(void);
unsigned char ucButtonPress, ucDACupdate = 0;  // Variables used to indicate and count XIRQ button press
int main (void)
{
   int i,j,k = 0;
   
   WdtCfg(T3CON_MOD_FREERUN,T3CON_PRE_DIV1,T3CON_IRQ_EN,T3CON_PMD_DIS);    // Disable the Watchdog timer resets
   
   if (pADI_ID->CHIPID == 0x561)                      // check for S2 silicon
   {
      pADI_RESET->RSTKEY =  0x2009;                   //By Default Software and Watchdog resets don't reset the LV die and GPIOs
      pADI_RESET->RSTKEY =  0x0426;                   //During debug it's useful that Software tools have the ability to reset them
      pADI_RESET->RSTCFG = RSTCFG_GPIO_PLA_RETAIN_DIS;//So disable retain function during debug.
   }
   if (pADI_LV->LVID == 0x73)                         // check for S2 silicon
   {
      pADI_LV_RST->LVRST = LVRST_RETAIN_DIS;
   }
   
   DioOenPin(pADI_GP2, PIN4, 1);               // Set P2.4 as an output to toggle the LED
   
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }

   DioPul(pADI_GP2, 0xFF);                     // Enable pull-up resistors on P2 pins
   DioPul(pADI_GP1, 0xFF);                     // Enable pull-up resistors on P1 pins
   DioPul(pADI_GP0, 0xFF);                     // Enable pull-up resistors on P0 pins
   
   pADI_CLKCTL->CLKCON5 = 0x00;                // Enable clock to all peripherals
   pADI_CLKCTL->CLKCON1 = 0x200;               // PCLK 20MHz, HCLK 80MHz
   VDACSetup();                                // Setup VDACs
   ucButtonPress = 0;
   ucDACupdate = 0;
   EiCfg(EXTINT0, INT_EN, INT_FALL);           // Enable External IRQ 0 
   NVIC_EnableIRQ(EINT0_IRQn);                 // Enable External IRQ0 interrupt source in NVIC
   while (1)
   {
      if (ucButtonPress == 1)
      {

         switch (ucDACupdate)
         {
            case 1:                           // Set VDACs to zero-scale */
            {
               DacWr(pADI_VDAC0,0x00000000);                        
               DacWr(pADI_VDAC1,0x00000000);                        
               DacWr(pADI_VDAC2,0x00000000);                        
               DacWr(pADI_VDAC3,0x00000000);                        
               DacWr(pADI_VDAC4,0x00000000);                        
               DacWr(pADI_VDAC5,0x00000000);  
               DacWr(pADI_VDAC6,0x00000000);                        
               DacWr(pADI_VDAC7,0x00000000);  
               break;                
            }
            case 2:                           // Set VDACs to 1/8-scale */
            {
               DacWr(pADI_VDAC0,0x01FF0000);                        
               DacWr(pADI_VDAC1,0x01FF0000);                        
               DacWr(pADI_VDAC2,0x01FF0000);                        
               DacWr(pADI_VDAC3,0x01FF0000);                        
               DacWr(pADI_VDAC4,0x01FF0000);                        
               DacWr(pADI_VDAC5,0x01FF0000);  
               DacWr(pADI_VDAC6,0x01FF0000);                        
               DacWr(pADI_VDAC7,0x01FF0000);     
               break;                  
            }
            case 3:                           // Set VDACs to 1/4-scale */
            {
               DacWr(pADI_VDAC0,0x03FF0000);                        
               DacWr(pADI_VDAC1,0x03FF0000);                        
               DacWr(pADI_VDAC2,0x03FF0000);                        
               DacWr(pADI_VDAC3,0x03FF0000);                        
               DacWr(pADI_VDAC4,0x03FF0000);                        
               DacWr(pADI_VDAC5,0x03FF0000);  
               DacWr(pADI_VDAC6,0x03FF0000);                        
               DacWr(pADI_VDAC7,0x03FF0000); 
               break;                  
            }
            case 4:                           // Set VDACs to 3/8-scale */
            {
               DacWr(pADI_VDAC0,0x05FF0000);                        
               DacWr(pADI_VDAC1,0x05FF0000);                        
               DacWr(pADI_VDAC2,0x05FF0000);                        
               DacWr(pADI_VDAC3,0x05FF0000);                        
               DacWr(pADI_VDAC4,0x05FF0000);                        
               DacWr(pADI_VDAC5,0x05FF0000);  
               DacWr(pADI_VDAC6,0x05FF0000);                        
               DacWr(pADI_VDAC7,0x05FF0000); 
               break;                  
            }
            case 5:                           // Set VDACs to 1/2-scale */
            {
               DacWr(pADI_VDAC0,0x07FF0000);                        
               DacWr(pADI_VDAC1,0x07FF0000);                        
               DacWr(pADI_VDAC2,0x07FF0000);                        
               DacWr(pADI_VDAC3,0x07FF0000);                        
               DacWr(pADI_VDAC4,0x07FF0000);                        
               DacWr(pADI_VDAC5,0x07FF0000);  
               DacWr(pADI_VDAC6,0x07FF0000);                        
               DacWr(pADI_VDAC7,0x07FF0000);   
               break;                  
            }
            case 6:                           // Set VDACs to 5/8-scale */
            {
               DacWr(pADI_VDAC0,0x09FF0000);                        
               DacWr(pADI_VDAC1,0x09FF0000);                        
               DacWr(pADI_VDAC2,0x09FF0000);                        
               DacWr(pADI_VDAC3,0x09FF0000);                        
               DacWr(pADI_VDAC4,0x09FF0000);                        
               DacWr(pADI_VDAC5,0x09FF0000);  
               DacWr(pADI_VDAC6,0x09FF0000);                        
               DacWr(pADI_VDAC7,0x09FF0000);   
               break;                  
            }
            case 7:                           // Set VDACs to 3/4-scale */
            {
               DacWr(pADI_VDAC0,0x0BFF0000);                        
               DacWr(pADI_VDAC1,0x0BFF0000);                        
               DacWr(pADI_VDAC2,0x0BFF0000);                        
               DacWr(pADI_VDAC3,0x0BFF0000);                        
               DacWr(pADI_VDAC4,0x0BFF0000);                        
               DacWr(pADI_VDAC5,0x0BFF0000);  
               DacWr(pADI_VDAC6,0x0BFF0000);                        
               DacWr(pADI_VDAC7,0x0BFF0000);  
               break;                  
            }  
            case 8:                           // Set VDACs to 7/8-scale */
            {
               DacWr(pADI_VDAC0,0x0DFF0000);                        
               DacWr(pADI_VDAC1,0x0DFF0000);                        
               DacWr(pADI_VDAC2,0x0DFF0000);                        
               DacWr(pADI_VDAC3,0x0DFF0000);                        
               DacWr(pADI_VDAC4,0x0DFF0000);                        
               DacWr(pADI_VDAC5,0x0DFF0000);  
               DacWr(pADI_VDAC6,0x0DFF0000);                        
               DacWr(pADI_VDAC7,0x0DFF0000);  
               break;                  
            } 
            case 9:                           // Set VDACs to full-scale */
            {
               DacWr(pADI_VDAC0,0x0FFF0000);                        
               DacWr(pADI_VDAC1,0x0FFF0000);                        
               DacWr(pADI_VDAC2,0x0FFF0000);                        
               DacWr(pADI_VDAC3,0x0FFF0000);                        
               DacWr(pADI_VDAC4,0x0FFF0000);                        
               DacWr(pADI_VDAC5,0x0FFF0000);  
               DacWr(pADI_VDAC6,0x0FFF0000);                        
               DacWr(pADI_VDAC7,0x0FFF0000);   
               break;                  
            }
            default :
            break; 
         }
         ucButtonPress = 0;
         delay(10000000);
         EiCfg(EXTINT0, INT_EN, INT_FALL);           // Enable External IRQ 0 
      }
   }
}

void VDACSetup(void)
{
   DacPwrDwn(pADI_VDAC0,0);                             // power up VDAC0
   DacPwrDwn(pADI_VDAC1,0);                             // power up VDAC1  
   DacPwrDwn(pADI_VDAC2,0);                             // power up VDAC2
   DacPwrDwn(pADI_VDAC3,0);                             // power up VDAC3   
   DacPwrDwn(pADI_VDAC4,0);                             // power up VDAC4
   DacPwrDwn(pADI_VDAC5,0);                             // power up VDAC5  
   DacPwrDwn(pADI_VDAC6,0);                             // power up VDAC6
   DacPwrDwn(pADI_VDAC7,0);                             // power up VDAC7

   DacCfg(pADI_VDAC0,DAC0CON_MODE_12BIT,INT_REF);     // Turn on VDAC0, and use internal reference
   DacCfg(pADI_VDAC1,DAC1CON_MODE_12BIT,INT_REF);     // Turn on VDAC1, and use internal reference
   DacCfg(pADI_VDAC2,DAC2CON_MODE_12BIT,INT_REF);     // Turn on VDAC2, and use internal reference
   DacCfg(pADI_VDAC3,DAC3CON_MODE_12BIT,INT_REF);     // Turn on VDAC3, and use internal reference
   DacCfg(pADI_VDAC4,DAC4CON_MODE_12BIT,INT_REF);     // Turn on VDAC4, and use internal reference
   DacCfg(pADI_VDAC5,DAC5CON_MODE_12BIT,INT_REF);     // Turn on VDAC5, and use internal reference
   DacCfg(pADI_VDAC6,DAC6CON_MODE_12BIT,INT_REF);     // Turn on VDAC6, and use internal reference
   DacCfg(pADI_VDAC7,DAC7CON_MODE_12BIT,INT_REF);     // Turn on VDAC7, and use internal reference
      
   DacWr(pADI_VDAC0,0x08000000);                // Set VDAC0 to mid-scale
   DacWr(pADI_VDAC1,0x04000000);                // Set VDAC1 to quarter-scale
   DacWr(pADI_VDAC2,0x0C000000);                // Set VDAC2 to 3/4-scale
   DacWr(pADI_VDAC3,0x08000000);                // Set VDAC3 to mid-scale 
   DacWr(pADI_VDAC7,0x08000000);                // Set VDAC7 to mid-scale
   DacWr(pADI_VDAC6,0x04000000);                // Set VDAC6 to quarter-scale
   DacWr(pADI_VDAC5,0x0C000000);                // Set VDAC5 to 3/4-scale
   DacWr(pADI_VDAC4,0x08000000);                // Set VDAC4 to mid-scale */
}

void Ext_Int0_Handler()
{
   DioTgl(pADI_GP2, BIT4);                      // Toggle P2.4
   EiClr(EXTINT0);                              // Clear the interrupt0 source
   EiCfg(EXTINT0, INT_DIS, INT_FALL);           // Disable External IRQ 0 

   ucButtonPress = 1;
   ucDACupdate++;
   if (ucDACupdate >= 10)
      ucDACupdate = 1;
}

// Simple Delay routine
void delay (long int length)
{
   while (length >0)
      length--;
}
