/**
 *****************************************************************************
   @addtogroup wut 
   @{
   @file     WutLib.c
   @brief    Set of wake up Timer peripheral functions.
   
   @internal   001 @endinternal
   @version  V1.0
   @author   ADI
   @date     April 2015
   @par Revision History:
   - V0.1, April 2013: initial version.
   - V0.2, May 2013: Fixed doxygen comments
   - V0.3, May 2013: Corrected WutLdRd() - correct values returned
                     Corrected WutLdWr() - T4WUFA is re-enabled after writing to it
   - V0.4, Oct 2013: Changed WutCfg() doxygen comment
   - V1.0, April 2015: Changed library format

All files provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include "WutLib.h"


/**
	@brief int WutCfg(int iMode, int iWake, int iPre, int iClkSrc)
			
	@param iMode :{T4CON_MOD_PERIODIC,T4CON_MOD_FREERUN}
		- 0 or T4CON_MOD_PERIODIC for periodic mode
		- 0x40 or T4CON_MOD_FREERUN for free running mode
	@param iWake :{T4CON_WUEN_DIS,T4CON_WUEN_EN}
		- 0 or T4CON_WUEN_DIS to use the wake up timer as general purpose timer
		- 0x100 or T4CON_WUEN_EN to allow the wake up timer to wake up the part
	@param iPre :{0,1,2,3}
		- 0 or T4CON_PRE_DIV1
		- 1 or T4CON_PRE_DIV16
      - 2 or T4CON_PRE_DIV256
      - 3 or T4CON_PRE_DIV32768
	@param iClkSrc :{T4CON_CLK_PCLK,T4CON_CLK_LFOSC,T4CON_CLK_ECLKIN}
		- 0 for T4CON_CLK_PCLK => Clocked by system clock.
		- 0x200 or T4CON_CLK_LFOSC =>  Clocked by 32kHz internal oscillator.
		- 0x600 or T4CON_CLK_ECLKIN =>  Clocked by external signal on GPIO.
	@return 0 if timer interface busy or 1 if successful.
	
**/

int WutCfg(int iMode, int iWake, int iPre, int iClkSrc)
{
  int	i1 = 0;

  i1 |= iMode;
  i1 |= iWake;
  i1 |= iClkSrc;
  i1 |= iPre;
  i1 |= T4CON_WUEN; // the wake up timer interrupt if enabled is allowed to wake up the device
  pADI_WUT->T4CON = i1;
  return 1;	
}


/**
	@brief int WutInc(int iInc);
			==========Sets timer increment value.
	@param iInc :{0-0xFFF}
      - Sets increment value TxLD to iInc.
      - if the timer is already running, this function stops WUA during update of increment register
	@return 1
**/
int WutInc(int iInc)
{
  if (T4CON_ENABLE_BBA == 1) T4CON_STOP_WUFA_BBA = 1;
  pADI_WUT->T4INC = iInc;
  T4CON_STOP_WUFA_BBA = 0;
  return 1;
}


/**
	@brief int WutLdWr(int iField, unsigned long lTld);
         ==========Sets timer reload value.
	@param iField :{0, 1, 2, 3}
      - 0 for field A
      - 1 for field B
      - 2 for field C
      - 3 for field D. In periodic mode field D is the timeout.
	@param lTld :{0-0xFFFFFFFF}
		- Sets reload value TxLD to lTLd. 
	@return 0 if invalid field or 1 if field is valid
   @note If the timer is already running the corresponding interrupt should be diabled.
**/
int WutLdWr(int iField, unsigned long lTld)
{      
  switch (iField)
  {
  case 0:
    {
      if (T4CON_ENABLE_BBA == 1) T4CON_STOP_WUFA_BBA = 1;
      pADI_WUT->T4WUFA0= (short)(lTld&0x0000FFFF);
      pADI_WUT->T4WUFA1= (short)(lTld>>16);
      T4CON_STOP_WUFA_BBA = 0;
      break;
    }
  
  case 1:
    {
      pADI_WUT->T4WUFB0= (short)(lTld&0x0000FFFF);
      pADI_WUT->T4WUFB1= (short)(lTld>>16);
      break;
    }
  case 2:
    {
      pADI_WUT->T4WUFC0= (short)(lTld&0x0000FFFF);
      pADI_WUT->T4WUFC1= (short)(lTld>>16);
      break;
    }
  case 3:
    {
      pADI_WUT->T4WUFD0= (short)(lTld&0x0000FFFF);
      pADI_WUT->T4WUFD1= (short)(lTld>>16);
      break;
    }  
  default:
    return 0;
 }
 return 1;	
}

/**
	@brief unsigned long WutLdRr(int iField);
			==========Sets timer reload value.
	@param iField :{0, 1, 2, 3}
		- 0 for field A
		- 1 for field B
		- 2 for field C
      - 3 for field D. 
	@return TxLD corresponding to iField or 0 if iField is not valid
**/

unsigned long WutLdRd(int iField)
{
  unsigned long uL;
  switch (iField)
  {
  case 0:
    {
      uL = pADI_WUT->T4WUFA1;
      uL = uL<<16;
      uL |= pADI_WUT->T4WUFA0;
      break;
    }  
  case 1:
    {
      uL = pADI_WUT->T4WUFB1;
      uL = uL<<16;
      uL |= pADI_WUT->T4WUFB0;
      break;
    }
  case 2:
    {
      uL = pADI_WUT->T4WUFC1;
      uL = uL<<16;
      uL |= pADI_WUT->T4WUFC0;
      break;
    }
  case 3:
    {
      uL = pADI_WUT->T4WUFD1;
      uL = uL<<16;
      uL |= pADI_WUT->T4WUFD0;
      break;
    }  
  default:
    {
      uL = 0;
      break;
    }
  }  
  return uL;
}

/**
	@brief long WutVal(void);
			==========Reads timer value.

	@return 0 if freeze has not worked...
                 or timer value T4VAL.
**/

long WutVal(void)
{
  long lVal;
  if (T4CON_FREEZE_BBA == 0) T4CON_FREEZE_BBA = 1;
  lVal = pADI_WUT->T4VAL0;
  if (T4STA_FREEZE_BBA == 0) return 0;  // T4VAL is not frozen
  lVal |= (pADI_WUT->T4VAL1<<16);
  return (lVal);
}


	
/**
	@brief int WutSta(void);
			==========Returns Timer Status.
	@return T4STA.
**/

int WutSta(void)
{
  return (pADI_WUT->T4STA);
}



/**
	@brief int WutClrInt(int iSource);
			==========Clear timer interrupts.
	@param iSource :{T4CLRI_WUFA,T4CLRI_WUFB,T4CLRI_WUFC,T4CLRI_WUFD,T4CLRI_ROLL}
		- 0x1 or T4CLRI_WUFA for wake up field A interrupt
		- 0x2 or T4CLRI_WUFB for wake up field B interrupt
      - 0x4 or T4CLRI_WUFC for wake up field C interrupt
      - 0x8 or T4CLRI_WUFD for wake up field D interrupt
      - 0x10 or T4CLRI_ROLL for timer overflow interrupt
	@return 1.
**/
int WutClrInt(int iSource)
{
  (pADI_WUT->T4CLRI) = iSource;
  return 1;
}


/**
   @brief int WutCfgInt(int iSource, iEnable);
			==========Clear timer interrupts.
   @param iSource :{T4IEN_WUFA| T4IEN_WUFB| T4IEN_WUFC| T4IEN_WUFD| T4IEN_ROLL}
      - 0x1 or T4IEN_WUFA for wake up field A interrupt
      - 0x2 or T4IEN_WUFB for wake up field B interrupt
      - 0x4 or T4IEN_WUFC for wake up field C interrupt
      - 0x8 or T4IEN_WUFD for wake up field D interrupt
      - 0x10 or T4IEN_ROLL for timer overflow interrupt
   @param iEnable :{0,1}
      - 0 disable interrupt
      - 1 enable interrupt
   @return 1.
**/
int WutCfgInt(int iSource, int iEnable)
{
	int iInterrupt = 0;

	if (iEnable == 1)				   // Enabling interrupts	
	{
		iInterrupt = (iSource & 0x1F);
		pADI_WUT->T4IEN |= iInterrupt;
	}
	else
	{
		iInterrupt = pADI_WUT->T4IEN;
		iInterrupt = (iSource ^ iInterrupt);
		pADI_WUT->T4IEN = (iInterrupt & 0x1F);		
	}
  	return 1;
}


/**
	@brief int WutGo(int iEnable);
			==========Enable or reset the wake up timer
	@param iEnable :{T4CON_ENABLE_DIS,T4CON_ENABLE_EN}
		- 0 or T4CON_ENABLE_DIS to reset the timer
		- 0x80 or T4CON_ENABLE_EN to enable the timer
	@return 1.
**/
int WutGo(int iEnable)
{
  if (iEnable == T4CON_ENABLE_DIS)
    pADI_WUT->T4CON = T4CON_ENABLE_DIS;   
  else
    T4CON_ENABLE_BBA = 1;
  return 1;
}


/**@}*/
