/**
 *****************************************************************************
   @example  ADC_DMA.c
   @brief    DC_DMA.c
      Decription: Demo for ADC in DMA mode.
      - Sets up ADC for continous sampling on AIN0 input channel in Single ended mode
      - 128x samples taken and stored direct to SRAM via DMA controller
      - After the first set of samples, the ADC is set up for continuos sampling on
         the next channel and the DMA is used to store the results as before.
      - UART configured for 115200-8-N-1 - results sent to UART 
         when DMA sequence completed
      - 2x VDACs setup as voltage sources also
      
  
   @version  V0.2
   @author   ADI
   @date     November 2015

All files for this device is provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>


#include <DmaLib.h>
#include <DacLib.h>
#include <WdtLib.h>
#include <UrtLib.h>
#include <DioLib.h>
#include <AdcLib.h>
// Function Definitions
void delay(long int);
void ResultsToUart(unsigned int ulVal, unsigned int ucChan);
void UartSetup(void);
void ADCDMASetUp(void);
void VDACSetup(void);
void DMAINIT(void);
// Global Variable definitions
volatile unsigned char ucSendToUart = 0; 
volatile unsigned char ucTxBufferEmpty = 0;	  // Indicates UART Tx buffer empty
volatile unsigned int uiTest = 0;
unsigned char szTemp[64] = "";		           // Used to store ADC1 result before printing to UART
volatile unsigned char ucCOMSTA0 = 0;          // Variable used to store COMSTA0, UART status register
volatile unsigned char ucCOMIID0 = 0;          // Variable used to store COMIID0, UART Interrupt status register	 
volatile unsigned char ucComRx = 0;            // Variable used to read UART Rx buffer contents into 

#define DMA_CYCLES (99)
#define numDmaTran  (128)
float fVoltage = 0;   			              // ADC value converted to voltage
float fVolts = 0.0;
unsigned int uiAdcChan = AIN0; 
unsigned int uxADCData[3][numDmaTran];
unsigned char ucDMA_Complete = 0;

int main (void)
{
   int i = 0, j = 0, k = 0;
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);    // Disable the Watchdog timer resets
   pADI_RESET->RSTKEY =  0x2009;                //By Default Software and Watchdog resets don't reset the LV die and GPIOs
   pADI_RESET->RSTKEY =  0x0426;               //During debug it's usefull that Software tools have the ability to reset them
   pADI_RESET->RSTCFG = 0x0;									 //So disable retain function during debug.
   pADI_LV_RST->LVRST = LVRST_RETAIN_DIS;

   DioOenPin(pADI_GP2,PIN4,1);                 // Configure P2.4 as an output - to toggle LED
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }

   UartSetup();
   VDACSetup();
   
   ucDMA_Complete = 0;
   ADCDMASetUp();   
   DMAINIT();                                  // Initialize DMA controller   
   DmaPeripheralStructSetup(ADC_C,DMA_DSTINC_WORD|DMA_SRCINC_NO|DMA_SIZE_WORD); 
   AdcDmaReadSetup(ADC_C,numDmaTran,uiAdcChan, uxADCData[0]);    
   DmaGo(ADC_C,numDmaTran,DMA_BASIC);
   DmaSet(0,ADC_B,0,0); 
   NVIC_EnableIRQ(DMA_ADC_IRQn);			           // Enable ADC DMA IRQ   
   DioSet(pADI_GP2, BIT4);
   j = 0;
   AdcGo(ADCCON_C_TYPE_CONT);

   while (1)
   {
      if (ucDMA_Complete == 1)
      {
         ucDMA_Complete = 0;
         if (j< DMA_CYCLES){
            ResultsToUart(uxADCData[uiAdcChan][0], uiAdcChan);
            //delay (0x80000);
            uiAdcChan++;
            uiAdcChan %= 3;
            AdcPin(VREFN_NADC,uiAdcChan);               
            AdcDmaReadSetup(ADC_C,numDmaTran,uiAdcChan, uxADCData[uiAdcChan]);    
            DmaGo(ADC_C,numDmaTran,DMA_BASIC);
            DmaSet(0,ADC_B,0,0);
            AdcGo(ADCCON_C_TYPE_CONT);
            j++;
         }
       }
   }
}

// This function converts ADC reading to a voltage and sends it to the UART
void ResultsToUart(unsigned int ulVal, unsigned int ucChan)
{
   unsigned int i = 0;
   unsigned int nLen = 0; 

   fVolts   = (2.51 / 65535);                 // Internal reference, calculate lsb size in volts
   fVoltage = ((ulVal>>12) * fVolts);          // Calculate ADC result in volts

   sprintf ( (char*)szTemp, "%fV,%x \r\n",fVoltage,ucChan );// Scan string with the ADC1 Result  
   nLen = strlen((char*)szTemp);               // Call function to calcualte the length of scanned string
   if (nLen <64)
   {
      for ( i = 0 ; i < nLen ; i++ )	        // loop to send ADC result	to UART
       {
          ucTxBufferEmpty = 0;	              // Clear flag
          UrtTx(pADI_UART,szTemp[i]);          // Load UART Tx register.
          while (ucTxBufferEmpty == 0)         // Wait for UART Tx interrupt
          {
          }
       }
   }
}

// This function initialises VDACs and sets their output
void VDACSetup(void)
{
   DacPwrDwn(pADI_VDAC2,0);                    // power up VDAC2
   DacPwrDwn(pADI_VDAC3,0);                    // power up VDAC3   
   DacCfg(pADI_VDAC2,
       DACCON_MODE_12BIT,INT_REF);             // Turn on VDAC2,  and use internal reference
   DacCfg(pADI_VDAC3,
       DACCON_MODE_12BIT,INT_REF);             // Turn on VDAC3,  and use internal reference
   DacWr(pADI_VDAC2,0x04000000);               // Set VDAC2 to 3/4-scale
   DacWr(pADI_VDAC3,0x08000000);               // Set VDAC3 to mid-scale
}

void DMAINIT(void)  
{
   DmaBase();                 
}

/* Function used to initialize ADC in DMA mode  */
void ADCDMASetUp(void)
{
   AdcBuf(IBUFCON_IBUF_PD_NONE,
   IBUFCON_IBUF_BYP_BOTH,ADCCON_REFB_PUP);     // Bypass and power down internal buffer
   
   if(pADI_ID->CHIPID == 0x560)  //S1
   {
      pADI_ADC->ADCCNVC = 0x800A0;                               // make sure top bits are correct for s1
      AdcSpeed(160);                               // Value for 100KSPS
   }
   else
   {
      pADI_ADC->ADCCNVC = 0xA00C8;                               // make sure top bits are correct for s2
      AdcSpeed(200);                               // Value for 100KSPS
   }
   AdcGo(ADCCON_C_TYPE_NO);                    // Power up ADC
   AdcPin(VREFN_NADC,uiAdcChan);               
   AdcDma(ADCCON_CNV_DMA);                      // ADC DMA - continuous conversions
}

// Function used to setup the UART block
void UartSetup(void)
{
   DioCfg(pADI_GP1,0x5);                       // Configure P1[1:0] as UART pins
   UrtCfg(pADI_UART,B115200,COMLCR_WLS_EIGHTBITS,0);// Configure UART for 115200 baud rate
   UrtIntCfg(pADI_UART,COMIEN_ERBFI|
   COMIEN_ETBEI|COMIEN_ELSI);                  // Enable Rx, Tx and Rx buffer full Interrupts
   NVIC_EnableIRQ(UART_IRQn);                  // Enable UART interrupt source in NVIC
}
// Simple Delay routine
void delay (long int length)
{
   while (length >0)
      length--;
}

void DMA_ADC_Int_Handler()
{
  ucDMA_Complete = 1; 
  AdcGo(ADCCON_C_TYPE_NO);
}

void UART_Int_Handler()
{  	
   ucCOMSTA0 = UrtLinSta(pADI_UART);
   ucCOMIID0 = UrtIntSta(pADI_UART);
   if ((ucCOMIID0 & 0x2) == 0x2)	              // Transmit buffer empty
   {
	  ucTxBufferEmpty = 1;
   }	
   if ((ucCOMIID0 & 0x4) == 0x4)	              // Receive byte
   {
	  ucComRx	= UrtRx(pADI_UART);
   }
}
