/**
 *****************************************************************************
   @example  ADC_SystemSeqDmaDCcodes.c
   @brief    ADC Sequencer Demo file
      Decription: 
      - Sets up ADC Sequencer for continous sampling in Single ended mode
		- Input buffer disabled
		- DMA operation enabled - Autorequest mode
      - UART configured for 115200-8-N-1
		- Result of sampling AIN4 is sent to UART

      
  
   @version  V0.2
	 @author   ADI
   @date     December 2015
	 @par      Revision History:
   - V0.1,   June 2013: initial version.
	 - V0.2,   December 2015: Changed ADuCM3xx.h to DeviceHeader.h
	

All files for the device are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>

#include <DacLib.h>
#include <WdtLib.h>
#include <GptLib.h>
#include <UrtLib.h>
#include <DioLib.h>
#include <AdcLib.h>
#include <DmaLib.h>

// Function Definitions

void ResultsToUart(unsigned long ulVal, unsigned char ucChan);
void UartSetup(void);
void ADCSetUp(void);
void ADCDMASetUp(void);
void DMAINIT(void);
void VDACSetup(void);


// Global Variable definitions
volatile unsigned char ucSeqComplete = 0; 
volatile unsigned char ucTxBufferEmpty = 0;	   // Indicates UART Tx buffer empty

unsigned char szTemp[64] = "";		             // Used to store ADC1 result before printing to UART
volatile unsigned char ucCOMSTA0 = 0;          // Variable used to store COMSTA0, UART status register
volatile unsigned char ucCOMIID0 = 0;          // Variable used to store COMIID0, UART Interrupt status register	 
volatile unsigned char ucComRx = 0;            // Variable used to read UART Rx buffer contents into 
volatile long ultest,ADCTEST1,ADCTEST2 = 0;
volatile unsigned char ucCharTcvd = 0;
volatile unsigned char ucAIN2Request = 0;

// Following are a list of variables used to read the ADC
volatile long ulAIN4Result = 0;
volatile unsigned int ucClipLErr = 0;          // Read ADCxDAT.1 status bit
volatile unsigned int ucClipHErr = 0;          // Read ADCxDAT.0 status bit
float  fVoltage = 0.0;   			                 // ADC value converted to voltage
float fVolts = 0.0;
volatile long ADC_INL[4];                      // Used for averaging ADC results 
volatile unsigned char x = 0;                  // Used to load Array of samples
long long llAverage = 0;

volatile long ulTempResult = 0;
float fTemp = 0.0;                             // For temp sensor measurement
unsigned char ucCycleNum = 0;
unsigned int ucSeqNum = 0;                     // Used to track the ADC sequence number
unsigned char ucUpdateVDACs = 0;
volatile unsigned int uiTest, uiCheck = 0;

// Following are for DMA usage
unsigned int uxADCData[32];
unsigned int numDmaTran = 0;
unsigned char ucWaitForSingleConv = 0;
int main (void)
{ 
   unsigned char ucLVIrqStatus = 0; 
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);    // Disable the Watchdog timer
   ucWaitForSingleConv = 0;
   pADI_RESET->RSTKEY =  0x2009;                // By Default Software and Watchdog resets don't reset the LV die and GPIOs
   pADI_RESET->RSTKEY =  0x0426;                // During debug it's usefull that Software tools have the ability to reset them
   pADI_RESET->RSTCFG = 0x0;									   //So disable retain function during debug.
   pADI_LV_RST->LVRST = 1;
	
	DioCfg(pADI_GP0,0x200); //P0.4 as debug output
   ucSeqNum = 0;
   ucUpdateVDACs = 0;
   DioOenPin(pADI_GP2,PIN4,1);                 // Configure P2.4 as an output
   DioSet(pADI_GP2, BIT4);
   DioOenPin(pADI_GP0,PIN7,1);                 // Configure P0.7 as an output
   DioSet(pADI_GP0, BIT7);
   DioOenPin(pADI_GP1,PIN5,1);                  // Configure P1.5 as an output
   DioSet(pADI_GP1, BIT5);
   pADI_CLKCTL->CLKCON5 = 0x00;                // Enable clock to all peripherals
   pADI_CLKCTL->CLKCON1 = 0xA00;               // PCLK 20MHz, HCLK 80MHz
   VDACSetup();
   UartSetup();                                // Call function to init UART for 115200 baud

   numDmaTran = 21;                            // Number of ADC sequence channels to be DMA'ed
   ADCDMASetUp();                              // Enable ADC in DMA mode and setup ADC DMA control structure
   DMAINIT();                                  // Initialize DMA controller   

   DmaPeripheralStructSetup(ADC_C,DMA_DSTINC_WORD|DMA_SRCINC_WORD|DMA_SIZE_WORD); 
   AdcDmaReadSetup(ADC_C,numDmaTran,20, uxADCData);	
   DmaGo(ADC_C,numDmaTran,DMA_AUTO);           // Enable ADC DMA channel - in AutoRequest mode
   DmaSet(0,ADC_B,0,0); 
   NVIC_EnableIRQ(DMA_ADC_IRQn);			         // Enable ADC DMA IRQ   
   pADI_ADC->ADCSEQ |= 0xC0000000;             // Enable Sequencer
   AdcGo(ADCCON_C_TYPE_CONT);

   AdcPin(VREFN_NADC,AIN4);                    // +ve = AIN4, -ve=VREFN - for single conversions between sequences 
   pADI_LV_INT->INTSEL = 0x01;                 // Enable ADC SW conversion LV1 interrupt sources
  //1 NVIC_EnableIRQ(LVD1_IRQn);			             // Enable LV1 IRQ
	 
   while (1)
   {
      if (ucSeqComplete == 1)                  // Flag set in ADC DMA interrupt handler
      {
         ADC_INL[x++] = uxADCData[4];
         if (x >=4)
         {
            x = 0;
            ucAIN2Request = 1;
         }
         ResultsToUart(uxADCData[4],4); 
         ucSeqComplete = 0;                   // Clear flag

         DmaGo(ADC_C,numDmaTran,DMA_AUTO);    // restart DMA sequence
         DmaSet(0,ADC_B,0,0);
         pADI_ADC->ADCSEQ |= 0xC0000000;
         pADI_ADC->ADCCON = 0x293;            // restart the ADC in continous conversion mode with DMA
      }
   }
}

// This function converts ADC reading to a voltage and sends it to the UART
void ResultsToUart(unsigned long ulVal, unsigned char ucChan)
{
   unsigned char i = 0;
   unsigned char nLen = 0; 

   ulVal = (ulVal >> 12);
	sprintf ( (char*)szTemp, "0x%X \r\n",ulVal );// Scan string with the ADC1 Result 
   nLen = strlen((char*)szTemp);                  // Call function to calcualte the length of scanned string
   if (nLen <64)
   {
      for ( i = 0 ; i < nLen ; i++ )	            // loop to send ADC result	to UART
      {
         ucTxBufferEmpty = 0;	                  // Clear flag
         UrtTx(pADI_UART,szTemp[i]);             // Load UART Tx register.
         while (ucTxBufferEmpty == 0)           // Wait for UART Tx interrupt
         {
         }
      }
      uiTest++;
      if (uiTest < 16384)
      {
         uiCheck = uiTest;
      }
      else
      {
         while (1){}           
      }
   }
}
// Function call to initialize DMA base pointer
void DMAINIT(void)                                 
{
   DmaBase();                 
}
/* Function used to initialize ADC in DMA mode  */
void ADCDMASetUp(void)
{
	AdcGo(ADCCON_C_TYPE_NO);                    // Power up ADC

//	pADI_ADC->ADCCNVC = 0xA00C8;               // 100KSPS (S2 silicon),
//	pADI_ADC->ADCCNVC = 0xA0014;                 // 1MSPS (S2 silicon),
	pADI_ADC->ADCCNVC = 0xA0028;                 // 500KSPS (S2 silicon),

   AdcBuf(IBUFCON_IBUF_PD_NONE,
   	IBUFCON_IBUF_BYP_BOTH,ADCCON_REFB_PUP);   // Buffer off

	
   // DMA for Sequencer
   AdcSeqCfg(ADCSEQ_EN,AIN0_Seq|AIN1_Seq|AIN2_Seq|
   AIN3_Seq|AIN4_Seq|AIN5_Seq|AIN6_Seq|AIN7_Seq|
   AIN8_Seq|AIN9_Seq|TEMP_Seq|VREFP_Seq|
   PVDD_Seq|IOVDD_2_Seq|AVDD_2_Seq,0xFF);
   AdcSeqDiffCfg(0x11,0x11,0x11,0x11);
   AdcDma(ADCCON_SEQ_DMA);
}

void VDACSetup(void)
{
   DacPwrDwn(pADI_VDAC2,0);                    // power up VDAC2
   DacPwrDwn(pADI_VDAC3,0);                    // power up VDAC3   
   DacCfg(pADI_VDAC2,
       DACCON_MODE_12BIT,INT_REF);             // Turn on VDAC2,  and use internal reference
   DacCfg(pADI_VDAC3,
       DACCON_MODE_12BIT,INT_REF);             // Turn on VDAC3,  and use internal reference
   DacWr(pADI_VDAC2,0x04000000);               // Set VDAC2 to 3/4-scale
   DacWr(pADI_VDAC3,0x08000000);               // Set VDAC3 to mid-scale
}

void ExternalIrqTest(void)
{
   pADI_GP0->GPCON = 0x40;
   pADI_INTERRUPT->EI0CFG = 0x8;               // External IRQ0 and IRQ1 and IRQ2 enabled
   NVIC_EnableIRQ(EINT0_IRQn);                 // Enable External interrupt 0 source
}

// Function used to setup the UART block
void UartSetup(void)
{
   DioCfg(pADI_GP1,0x5);                       // Configure P1[1:0] as UART pins
  // UrtCfg(pADI_UART,B9600,COMLCR_WLS_EIGHTBITS,0);// Configure UART for 9600 baud rate
   UrtCfg(pADI_UART,B115200,COMLCR_WLS_EIGHTBITS,0);// Configure UART for 115200 baud rate
   UrtIntCfg(pADI_UART,COMIEN_ERBFI|
   COMIEN_ETBEI|COMIEN_ELSI);                  // Enable Rx, Tx and Rx buffer full Interrupts
   NVIC_EnableIRQ(UART_IRQn);                  // Enable UART interrupt source in NVIC
}

void LV0_Int_Handler()
{
  
}
void DMA_ADC_Int_Handler()
{
  AdcGo(ADCCON_C_TYPE_NO);                     // Disable ADC conversions
  ucSeqComplete = 1;                           // Flag required in Main while loop
}
void LV1_Int_Handler()
{
   unsigned char ucLVIrqStatus = 0;
   
   ucLVIrqStatus = pADI_LV_INT->INTSTA;
   if ((ucLVIrqStatus & 0x1) == 0x1)           // ADC software conversion IRQ
   {
     pADI_LV_INT->INTCLR = 0x1;               // clear Irq source
     ucWaitForSingleConv = 0; 
		 ulAIN4Result = pADI_ADC->ADCDAT[4];
   }
   if ((ucLVIrqStatus & 0x2) == 0x2)           // ADC Sequencer IRQ
   {

   }
   if ((ucLVIrqStatus & 0x4) == 0x4)           // ADC Digital Comparator IRQ
   {
      pADI_LV_INT->INTCLR = 0x4;               // clear Irq source
   }
   if ((ucLVIrqStatus & 0x10) == 0x10)         
   {
      pADI_LV_INT->INTCLR = 0x10;              // clear Irq source
   }
   if ((ucLVIrqStatus & 0x20) == 0x20)         
   {
      pADI_LV_INT->INTCLR = 0x20;              // clear Irq source
   }
   if ((ucLVIrqStatus & 0x40) == 0x40)         // D2D read ECC error IRQ
   {
      pADI_LV_INT->INTCLR = 0x40;              // clear Irq source
   }
   if ((ucLVIrqStatus & 0x80) == 0x80)         // D2D Write ECC error IRQ
   {
      pADI_LV_INT->INTCLR = 0x80;              // clear Irq source
   }
}
void Ext_Int0_Handler()
{
   pADI_GP2->GPTGL = 0x10;                     // Toggle P2.4
   pADI_INTERRUPT->EICLR = 0x1;                // Clear the interrupt0 source
}

void UART_Int_Handler()
{  	
   ucCOMSTA0 = UrtLinSta(pADI_UART);
   ucCOMIID0 = UrtIntSta(pADI_UART);
   if ((ucCOMIID0 & 0x2) == 0x2)	              // Transmit buffer empty
   {
	  ucTxBufferEmpty = 1;
   }	
   if ((ucCOMIID0 & 0x4) == 0x4)	              // Receive byte
   {
	  ucCharTcvd	= UrtRx(pADI_UART);
      ucAIN2Request = 1;
   }
}

