/**
 *****************************************************************************
   @example  MdioEeprom.c
   @brief    Example code that implements 256 * 16 bit virtual EEPROM with an
             MDIO interface. The memory is initialised to 0x100 up to 0x2FF.
             To write to the memory send an address frame followed by a write frame.
             To read, send an address frame followed by a read frame.
             Post read increment frames are also supported.
             This project can be easily interacted with using SUB-20-LB-1.2 available xidimax
             and the software that comes with the dongle.
             
   @version  V0.2
   @author   ADI
   @date     December 2015
   @par Revision History:
   - V0.1, May 2014: initial version
	 - V0.2, December 2015: Changed ADuCM3xx.h to DeviceHeader.h

              
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include <stdio.h>
#include <string.h>
#include "DeviceHeader.h"
#include "MdLib.h"
#include "WdtLib.h"
#include "DioLib.h"
#include "UrtLib.h"

volatile unsigned short A0h[256];  
volatile unsigned int adr = 0;
volatile unsigned int count =0;
unsigned char szTemp[64] = "";		          // Used to store ADC1 result before printing to UART

int DioPulse(ADI_GPIO_TypeDef *pGP,int iBit,int iCnt);

volatile int iMdioSta;


int main(void)
{
   int i1;
   unsigned char nLen = 0; 

   WdtGo(T3CON_ENABLE_DIS);
	pADI_GP0->GPOE = 0x07;
   for(i1=0; i1<2000000; i1++)    DioPulse(pADI_GP0,2,1);
   
   for(i1 = 255; i1 >= 0; i1--)
   {
      A0h[i1] = i1+0x100;
   }
   
   DioPulse(pADI_GP0,2,1);
	DioPulse(pADI_GP0,2,1);
   DioCfgPin(pADI_GP3,PIN5, 1);
   DioCfgPin(pADI_GP3,PIN6, 1);

   pADI_GP3->GPCON = 0x0555;   //Put MDIO on P3. 
   MdInit(1,0,1);              //MDIO 5 bit with pushpull.
   MdCfg(5,0x1f,1);            //PHYADR = 5, DEVADR = 1. 
   MdTxd(0xa1ec);
   MdIen(MDIEN_MD_WRFI|MDIEN_MD_ADRI|MDIEN_MD_INCFI|MDIEN_MD_RDFI); 
   NVIC_EnableIRQ(MDIO_IRQn);  //Enable MDIO interrupt source in NVIC.
   DioCfg(pADI_GP1,0x5);                          // Configure P1[1:0] as UART pins
   UrtCfg(pADI_UART,B9600,COMLCR_WLS_EIGHTBITS,0);// Configure UART for 9600 baud rate
	while(1)
   {
      if(iMdioSta)
      {
         sprintf((char*)szTemp,"%08x\n\r",iMdioSta);
         nLen = strlen((char*)szTemp);                  // Call function to calcualte the length of scanned string
         if (nLen <64)
         {
            for ( i1 = 0 ; i1 < nLen ; i1++ )	          // loop to send String to UART
            {
               while(!(UrtLinSta(pADI_UART)&COMLSR_THRE));
               UrtTx(pADI_UART,szTemp[i1]);
            }
         }
         iMdioSta = 0;
      }
   }
}

int DioPulse(ADI_GPIO_TypeDef *pGP,int iBit,int iCnt)
    {
    int i1;

    for(i1=0; i1<iCnt; i1++)
        {
        if(pGP==pADI_GP1)   pADI_GPIO_FIX->GP1SET = 1<<iBit;
        else    pGP->GPSET = 1<<iBit;
        if(pGP==pADI_GP2)   pADI_GPIO_FIX->GP2CLR = 1<<iBit;
        else    pGP->GPCLR = 1<<iBit;
        }
    return 1;
    }
    
void MDIO_Int_Handler()
{
   //DioPulse(pADI_GP0,1,1);
   iMdioSta = MdSta();
   count++;
   if((iMdioSta & BIT1) == BIT1)    //end of adddress frame
   {
      //at the end of the address frame read the received address and preload
      // TX register in case next frame is a read frame
      //DioPulse(pADI_GP0,2,2);
      adr = MdAdr();
      iMdioSta += adr<<16;   //save address in top bits so that it can be printed over UART
      if(adr < 256){  //don't index past end of array
         MdTxd(A0h[adr]);
      }
      else{
         MdTxd(0xAAAA);
      }
      
   }
   if((iMdioSta & BIT0) == BIT0)   //end of write frame
   {
//      DioPulse(pADI_GP0,2,2);
      iMdioSta += MdRxd()<<16;   //save received data in top bits so that they can be printed over UART
      if(adr < 256){  //don't index past end of array
         A0h[adr] = MdRxd();
      }
   }
   
   if((iMdioSta & BIT2) == BIT2)  //end of post inc
   {
//      DioPulse(pADI_GP0,2,2);
      if(adr < 256){  //don't index past end of array
         MdTxd(A0h[adr]);
         iMdioSta += (A0h[adr]<<16);   //save sent data in top bits so that they can be printed over UART

      }
      else{
         MdTxd(0xAAAA);
         iMdioSta += 0xAAAA0000;  //save sent data in top bits so that they can be printed over UART
      }
      adr++;
   }
   if((iMdioSta & BIT3) == BIT3)  // end of read frame
   {
      iMdioSta += (A0h[adr]<<16);   //save sent data in top bits so that they can be printed over UART
   }
   if((iMdioSta & BIT4) == BIT4)  //DEVADD match
   {
   }
   if((iMdioSta & BIT5) == BIT5)  //DEVADD not matched
   {
   }
   if((iMdioSta & BIT6) == BIT6)  //PHYADR match
   {
   }
   if((iMdioSta & BIT5) == BIT5)  //PHYADR not matched
   {
   }

}
