/**
 *****************************************************************************
   @example  SPIDMA_Master_Spi0.c
   @brief    SPI0 master DMA example
      32-byte array is transmitted at the start and then another 32 bytes
      are transmitted everytime EINT0 is asserted. The primary DMA
      structures are used

   @version  V0.4
   @author   ADI
   @date     December 2015
   @par Revision History:
   - V0.1, July 2013: Initial release
   - V0.2, May 2014: SPI DMA needs to be disabled after interrupt and then re-enabled
   - V0.3, August 2015: Corrected typos in the code which called SPI1 instead of SPI0
	 - V0.4, December 2015: Changed ADuCM3xx.h to DeviceHeader.h

   
All files are provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.


**/

#include <stdio.h>
#include <string.h>
#include <DeviceHeader.h>


//#include <ClkLib.h>
#include <IntLib.h>
#include <DioLib.h>
#include <WdtLib.h>
#include <DioLib.h>
#include <DmaLib.h>
#include <SpiLib.h>

void delay(long int);
void SPI0INIT(void);
void DMAINIT(void);


unsigned int done_cnt = 0;
int dma_done = 0;

unsigned char ucIrqCnt = 0;                        // Used to count number of SPI0 DMA interrupts
unsigned char ucSendData = 0;                      // Flag used to setup 2nd and subsequent transfers
unsigned char uxSPI0WrData[]={0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2A,0x2B,0x2C,0x2D,0x2E,0x2F, 
                              0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F};
unsigned char uxSPI0RdData[40] = {0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 
                                  0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55, 0x55};

int main (void)
{
   unsigned int numDmaTran = 0;
   int i,j,k;
   WdtCfg(0,T3CON_PRE_DIV1,T3CON_IRQ_EN,0);                                   // Disable the Watchdog timer resets
   
   DioOenPin(pADI_GP2,PIN4,1);                                                // Pin  2.4 configured as output
   //delay before program starts
   for (i = 0; i<100; i++)
   {
      DioTgl(pADI_GP2, BIT4);       // Toggle LED, P2.4
      for (j = 0; j<100000; j++) k = pADI_GP2->GPIN;
   }

   DioOenPin(pADI_GP2,PIN6, 1);                                               // Set P2.5 as an output to toggle the LED
   
   DMAINIT();                                                                 // Initialize DMA controller
   SPI0INIT();                                                                // Initialize SPI0 block      
   
   EiCfg(EXTINT0,INT_EN,INT_RISE);                                            // Enable EINT0 - used to triger transfer
   NVIC_EnableIRQ(EINT0_IRQn);                                                // Enable IRQ0 interrupt
   NVIC_EnableIRQ(DMA_SPI0_TX_IRQn);                                          // SPI0 Tx DMA interrupt enable
     
   numDmaTran = 16;
   DmaStructPtrOutSetup(SPI0TX_C,numDmaTran,uxSPI0WrData);     // Setup source/destination pointer fields
   DmaGo(SPI0TX_C,numDmaTran,DMA_BASIC);                       // Enable SPI0TX DMA structure
   
   DmaStructPtrInSetup(SPI0RX_C,numDmaTran,uxSPI0RdData);      // Setup source/destination pointer fields
   DmaGo(SPI0RX_C,numDmaTran,DMA_BASIC);                       // Enable SP01RX DMA structure
   DmaSet(0,SPI0TX_B | SPI0RX_B,0,0);                          // Enable DNA structures for SPI0 TX and RX
   SpiCount(pADI_SPI0, numDmaTran*2, 1);                                      // Number of bytes the SPI should transfer (2* number of DMA transfers)
   SpiDma(pADI_SPI0,SPIDMA_IENRXDMA_EN,SPIDMA_IENTXDMA_EN,SPIDMA_ENABLE_EN);  // Enable SPI0 DMA operation - Tx
   
   while (1)
   {
   
      DioTgl(pADI_GP2,BIT4);   // Toggle P2.4
      delay(0x60000);         // Delay routine
      if (dma_done ==1 & ucSendData == 1)
      {
         ucSendData = 0;
         dma_done = 0;
         // After the DMA interrupt occurs and before setting up the DMA for the next SPI sequence we need to disable
         // the SPI DMA mode
         SpiDma(pADI_SPI0,SPIDMA_IENRXDMA_DIS,SPIDMA_IENTXDMA_DIS,SPIDMA_ENABLE_DIS);   

         DmaStructPtrOutSetup(SPI0TX_C,numDmaTran,uxSPI0WrData);  // Setup source/destination pointer fields
         DmaGo(SPI0TX_C,numDmaTran,DMA_BASIC);                    // Enable SPI0TX DMA structure

         DmaStructPtrInSetup(SPI0RX_C,numDmaTran,uxSPI0RdData);
         DmaGo(SPI0RX_C,numDmaTran,DMA_BASIC);

         DmaSet(0,SPI0TX_B| SPI0RX_B,0,0);

         SpiCount(pADI_SPI0, numDmaTran*2, 1);

         SpiDma(pADI_SPI0,SPIDMA_IENRXDMA_EN,SPIDMA_IENTXDMA_EN,SPIDMA_ENABLE_EN);
      }
   }
}

void SPI0INIT(void)
{                    
   DioCfgPin(pADI_GP0, PIN0, 1);                               // Configure P0[3:0] for SPI0
   DioCfgPin(pADI_GP0, PIN1, 1);
   DioCfgPin(pADI_GP0, PIN2, 1);
   DioCfgPin(pADI_GP0, PIN3, 1);
   SpiBaud(pADI_SPI0,0x3F,SPIDIV_BCRST_DIS);                   // Confiure SPI0 baud rate
   SpiCfg(pADI_SPI0,SPICON_MOD_TX1RX1,SPICON_MASEN_EN,
      SPICON_CON_DIS|SPICON_TIM_TXWR|SPICON_ENABLE_EN);        // Enable SPI0 master mode, continuous transfer, start on write to Tx

   DmaPeripheralStructSetup(SPI0TX_C,DMA_DSTINC_NO|
      DMA_SRCINC_HWORD|DMA_SIZE_HWORD);                        // Setup SPI0 Tx DMA structure
   
   DmaPeripheralStructSetup(SPI0RX_C,DMA_DSTINC_HWORD|
      DMA_SRCINC_NO|DMA_SIZE_HWORD);                           // Setup SPI0 Rx DMA structure
}
void DMAINIT(void)  
{
   DmaBase();                 
}
// Simple Delay routine
void delay (long int length)
{
   while (length >0)
      length--;
}
void WakeUp_Int_Handler(void)
{
  
}

void Ext_Int2_Handler ()
{  

}       
void Ext_Int0_Handler ()
{         
   EiClr(EXTINT0);                                          // Clear EINT0 interrupt flag
   ucSendData = 1;
}

void UART_Int_Handler ()
{

} 
void DMA_SPI0_TX_Int_Handler()
{
   DioTgl(pADI_GP2,BIT4);   // Toggle P2.4
   done_cnt ++;
   dma_done = 1;
}


void SPI0_Int_Handler()
{
   unsigned int SPI0MSTATUS = 0;
   
   SPI0MSTATUS = SpiSta(pADI_SPI0);               // Read SPI0STA register
}
